@@ -1,5 +1,5 @@
 /* Gnome Music Player Client (GMPC)
- * Copyright (C) 2004-2009 Qball Cow <qball@sarine.nl>
+ * Copyright (C) 2004-2010 Qball Cow <qball@sarine.nl>
  * Project homepage: http://gmpc.wikia.com/
  
  * This program is free software; you can redistribute it and/or modify
@@ -27,6 +27,7 @@
 #include "misc.h"
 #include "playlist3.h"
 #include "playlist3-current-playlist-browser.h"
+#include "playlist3-find2-browser.h"
 #include "gmpc-mpddata-model.h"
 #include "gmpc-mpddata-model-playlist.h"
 #include "gmpc-mpddata-treeview.h"
@@ -46,7 +47,7 @@ enum {
     PLAY_QUEUE_UID
 };
 /**
- * Private data structure 
+ * Private data structure
  */
 typedef struct _PlayQueuePluginPrivate {
     gulong status_changed_handler;
@@ -126,7 +127,7 @@ static void pl3_current_playlist_browser_init(PlayQueuePlugin *self);
 
 static void pl3_cp_current_song_changed(GmpcMpdDataModelPlaylist *model2,GtkTreePath *path, GtkTreeIter *iter,PlayQueuePlugin *self)
 {
-    GtkTreeModel *model; 
+    GtkTreeModel *model;
     if(self->priv->pl3_cp_tree == NULL) return;
     model = gtk_tree_view_get_model(GTK_TREE_VIEW(self->priv->pl3_cp_tree));
     if(GMPC_IS_MPDDATA_MODEL_PLAYLIST(model))
@@ -151,8 +152,8 @@ static void __real_pl3_total_playtime_changed(GmpcMpdDataModelPlaylist *model, u
          GString *tstring = g_string_new("");
 
          g_string_append_printf(tstring, "%lu %s",
-                 total_songs, 
-                 ngettext("item", "items", total_songs ) 
+                 total_songs,
+                 ngettext("item", "items", total_songs )
                  );
          if(string)
          {
@@ -186,7 +187,7 @@ static void pl3_current_playlist_browser_crop_current_song(PlayQueuePlugin *self
     mpd_Song *song = mpd_playlist_get_current_song(connection);
     if(song)
     {
-        int length = mpd_playlist_get_playlist_length(connection); 
+        int length = mpd_playlist_get_playlist_length(connection);
         /* Move to first place, this avoid possible "issues" */
         mpd_playlist_move_id(connection,song->id, 0);
         for(;length > 1; length--){
@@ -198,10 +199,14 @@ static void pl3_current_playlist_browser_crop_current_song(PlayQueuePlugin *self
         mpd_playlist_queue_commit(connection);
     }
 }
+static void pl3_cp_ec_playlist(PlayQueuePlugin *self, const gchar *param)
+{
+    pl3_find2_select_plugin_id(GMPC_PLUGIN_BASE(self)->id);
+    pl3_find2_do_search_any(param);
+}
+
 static void pl3_cp_init(PlayQueuePlugin *self)
 {
-
-//    pl3_current_playlist_browser_init(self);
     g_signal_connect(G_OBJECT(playlist), "current_song_changed", G_CALLBACK(pl3_cp_current_song_changed), self);
     g_signal_connect(G_OBJECT(playlist), "total_playtime_changed", G_CALLBACK(pl3_total_playtime_changed), self);
 
@@ -209,16 +214,21 @@ static void pl3_cp_init(PlayQueuePlugin *self)
     gmpc_easy_command_add_entry(gmpc_easy_command,
                     _("switch play queue"),"",
                     _("Switch to play queue"),
-                    (GmpcEasyCommandCallback *)pl3_current_playlist_browser_activate, self); 
+                    (GmpcEasyCommandCallback *)pl3_current_playlist_browser_activate, self);
     gmpc_easy_command_add_entry(gmpc_easy_command,
                     _("Clear play queue"),"",
                     _("Clear play queue"),
-                    (GmpcEasyCommandCallback *)pl3_current_playlist_browser_clear_playlist, self); 
+                    (GmpcEasyCommandCallback *)pl3_current_playlist_browser_clear_playlist, self);
 
     gmpc_easy_command_add_entry(gmpc_easy_command,
                     _("Crop current song"),"",
                     _("Crop the playlist so it only contains the current song"),
-                    (GmpcEasyCommandCallback *)pl3_current_playlist_browser_crop_current_song, self); 
+                    (GmpcEasyCommandCallback *)pl3_current_playlist_browser_crop_current_song, self);
+
+    gmpc_easy_command_add_entry(gmpc_easy_command,
+            _("search playlist"), ".*",
+            _("Search playlist <query>"),
+                (GmpcEasyCommandCallback *)pl3_cp_ec_playlist, self);
 }
 void pl3_current_playlist_destroy(PlayQueuePlugin *self);
 
@@ -243,35 +253,32 @@ static void pl3_current_playlist_column_changed(GtkTreeView *tree)
 
 void pl3_current_playlist_destroy(PlayQueuePlugin *self)
 {
-	if(self->priv->pl3_cp_tree)
-	{
-        /* destroy the entry */ 
-        if(self->priv->pl3_curb_tree_ref)
-		{
-			GtkTreeIter piter;
-			GtkTreePath *path;
-			path = gtk_tree_row_reference_get_path(self->priv->pl3_curb_tree_ref);
-			if(path)
-			{
-				if(gtk_tree_model_get_iter(GTK_TREE_MODEL(gtk_tree_row_reference_get_model(self->priv->pl3_curb_tree_ref)), &piter,path))
-				{
-					gtk_list_store_remove(GTK_LIST_STORE(gtk_tree_row_reference_get_model(self->priv->pl3_curb_tree_ref)), &piter);
-				}
-				gtk_tree_path_free(path);
-			}
-			gtk_tree_row_reference_free(self->priv->pl3_curb_tree_ref);
-			self->priv->pl3_curb_tree_ref = NULL;
-		}
-		/* destroy the browser */
-		if(self->priv->pl3_cp_vbox)
-		{
-			/* remove the signal handler so when the widget is destroyed, the numbering of the labels are not changed again */
-			g_signal_handlers_disconnect_by_func(G_OBJECT(self->priv->pl3_cp_tree), G_CALLBACK(pl3_current_playlist_column_changed), NULL);
-			g_object_unref(self->priv->pl3_cp_vbox);
-			self->priv->pl3_cp_vbox = NULL;
-		}
-		self->priv->pl3_cp_tree =  NULL;
-	}
+    /* destroy the entry */
+    if(self->priv->pl3_curb_tree_ref)
+    {
+        GtkTreeIter piter;
+        GtkTreePath *path;
+        path = gtk_tree_row_reference_get_path(self->priv->pl3_curb_tree_ref);
+        if(path)
+        {
+            if(gtk_tree_model_get_iter(GTK_TREE_MODEL(gtk_tree_row_reference_get_model(self->priv->pl3_curb_tree_ref)), &piter,path))
+            {
+                gtk_list_store_remove(GTK_LIST_STORE(gtk_tree_row_reference_get_model(self->priv->pl3_curb_tree_ref)), &piter);
+            }
+            gtk_tree_path_free(path);
+        }
+        gtk_tree_row_reference_free(self->priv->pl3_curb_tree_ref);
+        self->priv->pl3_curb_tree_ref = NULL;
+    }
+    /* destroy the browser */
+    if(self->priv->pl3_cp_vbox)
+    {
+        /* remove the signal handler so when the widget is destroyed, the numbering of the labels are not changed again */
+        g_signal_handlers_disconnect_by_func(G_OBJECT(self->priv->pl3_cp_tree), G_CALLBACK(pl3_current_playlist_column_changed), NULL);
+        g_object_unref(self->priv->pl3_cp_vbox);
+        self->priv->pl3_cp_vbox = NULL;
+    }
+    self->priv->pl3_cp_tree =  NULL;
 }
 
 
@@ -339,9 +346,15 @@ static gboolean mod_fill_entry_key_press_event(GtkWidget *entry, GdkEventKey *ev
 
 static void  mod_fill_entry_changed(GtkWidget *entry, PlayQueuePlugin *self)
 {
-    if(self->priv->quick_search_timeout != 0)
+    if(self->priv->quick_search_timeout != 0){
         g_source_remove(self->priv->quick_search_timeout);
-    self->priv->quick_search_timeout = g_timeout_add(250, (GSourceFunc)mod_fill_do_entry_changed, self);
+        self->priv->quick_search_timeout = 0;
+    }
+
+    if(cfg_get_single_value_as_int_with_default(config, "general", "search-as-you-type", 0) == 1)
+    {
+        self->priv->quick_search_timeout = g_timeout_add(250, (GSourceFunc)mod_fill_do_entry_changed, self);
+    }
     gtk_widget_show(entry);
 }
 static void mod_fill_entry_activate(GtkWidget *entry, PlayQueuePlugin *self)
@@ -419,7 +432,7 @@ static void pl3_current_playlist_browser_init(PlayQueuePlugin *self)
 
 	/* set initial state */
     self->priv->pl3_cp_tree = tree;
-	g_object_ref(G_OBJECT(self->priv->pl3_cp_vbox));
+	g_object_ref_sink(G_OBJECT(self->priv->pl3_cp_vbox));
 }
 
 static void pl3_current_playlist_browser_select_current_song(PlayQueuePlugin *self)
@@ -1091,7 +1104,10 @@ static void pl3_current_playlist_browser_activate(PlayQueuePlugin *self)
     gtk_widget_grab_focus(self->priv->pl3_cp_tree);
 }
 
-
+static void pl3_current_playlist_do_playlist_search(GmpcPluginBase *self)
+{
+    pl3_find2_select_plugin_id(self->id);
+}
 static int pl3_current_playlist_browser_add_go_menu(GmpcPluginBrowserIface *obj, GtkMenu *menu)
 {
     GtkWidget *item = NULL;
@@ -1103,7 +1119,15 @@ static int pl3_current_playlist_browser_add_go_menu(GmpcPluginBrowserIface *obj,
     gtk_widget_add_accelerator(GTK_WIDGET(item), "activate", gtk_menu_get_accel_group(GTK_MENU(menu)), GDK_F1, 0, GTK_ACCEL_VISIBLE);
     g_signal_connect_swapped(G_OBJECT(item), "activate", 
             G_CALLBACK(pl3_current_playlist_browser_activate), obj);
-    return 1;
+
+    item = gtk_image_menu_item_new_with_label(_("Search Playlist"));
+    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), 
+            gtk_image_new_from_icon_name("gtk-find", GTK_ICON_SIZE_MENU));
+    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+    gtk_widget_add_accelerator(GTK_WIDGET(item), "activate", gtk_menu_get_accel_group(GTK_MENU(menu)), GDK_j, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
+    g_signal_connect_swapped(G_OBJECT(item), "activate", 
+            G_CALLBACK(pl3_current_playlist_do_playlist_search), obj);
+    return 2;
 }
 
 static void pl3_current_playlist_connection_changed(GmpcConnection *conn, MpdObj *mi, int connect,PlayQueuePlugin *self)
@@ -1123,7 +1147,6 @@ static void pl3_current_playlist_save_myself(GmpcPluginBase *obj)
         if(path)
         {
             gint *indices = gtk_tree_path_get_indices(path);
-            debug_printf(DEBUG_INFO,"Saving myself to position: %i\n", indices[0]);
             cfg_set_single_value_as_int(config, "current-playlist","position",indices[0]);
             gtk_tree_path_free(path);
         }
@@ -1200,7 +1223,7 @@ static GObject *play_queue_plugin_constructor(GType type, guint n_construct_prop
 	GMPC_PLUGIN_BASE(self)->plugin_type = GMPC_PLUGIN_PL_BROWSER|GMPC_INTERNALL;
 	pl3_cp_init(self);
 
-	return G_OBJECT(self);
+    return G_OBJECT(self);
 }
 /**
  * Base GmpcPluginBase class
@@ -1239,7 +1262,42 @@ static void play_queue_browser_iface_init(GmpcPluginBrowserIfaceIface * iface) {
 static void play_queue_plugin_tool_menu_iface_init (GmpcPluginToolMenuIfaceIface * iface) {
 	iface->tool_menu_integration = pl3_current_playlist_tool_menu_integration;
 }
+/* Integrate Search */
+static gboolean play_queue_plugin_is_field_supported(GmpcPluginIntegrateSearchIface *obj, mpd_TagItems tag)
+{
+    if(tag == MPD_TAG_NUM_OF_ITEM_TYPES) return TRUE; 
+    return mpd_server_tag_supported(connection, tag);
+}
 
+static MpdData * play_queue_plugin_is_search(GmpcPluginIntegrateSearchIface *obj, mpd_TagItems num_field , const gchar *search_string)
+{
+    MpdData *data_t = NULL;
+    if(num_field == MPD_TAG_NUM_OF_ITEM_TYPES){
+        data_t = advanced_search(search_string, TRUE);
+    }else{
+        gchar ** splitted = tokenize_string(search_string);
+        int i =0;
+        gboolean found = FALSE;
+        for(i=0;splitted && splitted[i];i++) {                                                        
+            if(!found) {
+                mpd_playlist_search_start(connection, FALSE);
+                found = TRUE;
+            }
+            mpd_playlist_search_add_constraint(connection, num_field, splitted[i]);
+        }
+        if(splitted)
+            g_strfreev(splitted);
+        if(found) {
+            data_t = mpd_playlist_search_commit(connection);
+        }
+    }
+    return data_t;
+}
+
+static void play_queue_plugin_is_iface_init (GmpcPluginIntegrateSearchIfaceIface * iface) {
+    iface->field_supported = play_queue_plugin_is_field_supported;
+    iface->search = play_queue_plugin_is_search;
+}
 
 GType play_queue_plugin_get_type(void) {
 	static GType play_queue_plugin_type_id = 0;
@@ -1250,16 +1308,21 @@ GType play_queue_plugin_get_type(void) {
 			.instance_size = sizeof(PlayQueuePlugin),
 			.n_preallocs = 0
 		};
-        static const GInterfaceInfo iface_info = { 
-            (GInterfaceInitFunc) play_queue_browser_iface_init, 
+        static const GInterfaceInfo iface_info = {
+            (GInterfaceInitFunc) play_queue_browser_iface_init,
             (GInterfaceFinalizeFunc) NULL, NULL};
-        static const GInterfaceInfo iface_tm_info = { 
-            (GInterfaceInitFunc) play_queue_plugin_tool_menu_iface_init, 
+        static const GInterfaceInfo iface_tm_info = {
+            (GInterfaceInitFunc) play_queue_plugin_tool_menu_iface_init,
+            (GInterfaceFinalizeFunc) NULL, NULL};
+
+        static const GInterfaceInfo iface_is_info = {
+            (GInterfaceInitFunc) play_queue_plugin_is_iface_init,
             (GInterfaceFinalizeFunc) NULL, NULL};
         play_queue_plugin_type_id = g_type_register_static(GMPC_PLUGIN_TYPE_BASE, "PlayQueuePlugin", &info, 0);
 
 		g_type_add_interface_static (play_queue_plugin_type_id, GMPC_PLUGIN_TYPE_BROWSER_IFACE, &iface_info);
 		g_type_add_interface_static (play_queue_plugin_type_id, GMPC_PLUGIN_TYPE_TOOL_MENU_IFACE, &iface_tm_info);
+		g_type_add_interface_static (play_queue_plugin_type_id, GMPC_PLUGIN_TYPE_INTEGRATE_SEARCH_IFACE, &iface_is_info);
 	}
 	return play_queue_plugin_type_id;
 }
