@@ -1,5 +1,5 @@
 /* Gnome Music Player Client (GMPC)
- * Copyright (C) 2004-2009 Qball Cow <qball@sarine.nl>
+ * Copyright (C) 2004-2010 Qball Cow <qball@sarine.nl>
  * Project homepage: http://gmpc.wikia.com/
  
  * This program is free software; you can redistribute it and/or modify
@@ -25,10 +25,19 @@
 #include "misc.h"
 #include "playlist3.h"
 #include "playlist3-file-browser.h"
+#include "playlist3-find2-browser.h"
+#include "advanced-search.h"
 #include "gmpc-mpddata-model.h"
 #include "gmpc-mpddata-treeview.h"
 #include "playlist3-playlist-editor.h"
-#include "gmpc-paned-size-group.h"
+#include "gmpc-extras.h"
+
+#define LOG_DOMAIN "FileBrowser"
+
+static void pl3_file_browser_plugin_init(void);
+
+static gboolean pl3_file_browser_is_field_supported(int tag);
+static MpdData * pl3_file_browser_is_search(const int num_field ,const gchar *search_string, GError **error);
 
 static void pl3_file_browser_destroy(void);
 static void pl3_file_browser_add(GtkWidget *cat_tree);
@@ -92,12 +101,15 @@ gmpcPlBrowserPlugin file_browser_gbp = {
 	.selected       = pl3_file_browser_selected,
 	.unselected     = pl3_file_browser_unselected,
 	.add_go_menu    = pl3_file_browser_add_go_menu,
+    .integrate_search_field_supported = pl3_file_browser_is_field_supported,
+    .integrate_search   = pl3_file_browser_is_search
 };
 
 gmpcPlugin file_browser_plug = {
-	.name = 						N_("File Browser"),
+	.name = 						N_("Database"),
 	.version = 						{1,1,1},
 	.plugin_type = 					GMPC_PLUGIN_PL_BROWSER|GMPC_INTERNALL,
+    .init =                         pl3_file_browser_plugin_init,
   	.destroy = 						pl3_file_browser_destroy,
 	.browser = 						&file_browser_gbp,
 	.mpd_status_changed = 			pl3_file_browser_status_changed,
@@ -157,22 +169,22 @@ static void pl3_file_browser_init(void)
     sw = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_ETCHED_IN);
-    pl3_fb_dir_tree = tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL(pl3_fb_dir_store));
-    gtk_tree_view_set_enable_tree_lines(GTK_TREE_VIEW(tree), TRUE);
-    gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(tree), TRUE);
+	pl3_fb_dir_tree = tree = gtk_tree_view_new_with_model (GTK_TREE_MODEL(pl3_fb_dir_store));
+	gtk_tree_view_set_enable_tree_lines(GTK_TREE_VIEW(tree), TRUE);
+	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(tree), TRUE);
 	column = gtk_tree_view_column_new();
 	gtk_tree_view_column_set_title(column, _("Directories"));
 	renderer = gtk_cell_renderer_pixbuf_new();
 	gtk_tree_view_column_pack_start(column, renderer, FALSE);
-    gtk_tree_view_set_fixed_height_mode(GTK_TREE_VIEW(tree), TRUE);
-    gtk_tree_view_column_add_attribute(column, renderer, "icon-name",PL3_FB_ICON);
-    gtk_tree_view_column_set_sizing(column , GTK_TREE_VIEW_COLUMN_FIXED);
+	gtk_tree_view_column_add_attribute(column, renderer, "icon-name",PL3_FB_ICON);
+	gtk_tree_view_column_set_sizing(column , GTK_TREE_VIEW_COLUMN_FIXED);
 	renderer = gtk_cell_renderer_text_new();
 	gtk_tree_view_column_pack_start(column, renderer, TRUE);
 	gtk_tree_view_column_add_attribute(column, renderer, "text", PL3_FB_NAME);
 	gtk_tree_view_insert_column(GTK_TREE_VIEW(tree),column, -1);
 
-    gtk_tree_view_set_search_column(GTK_TREE_VIEW(pl3_fb_dir_tree), PL3_FB_NAME);
+	gtk_tree_view_set_fixed_height_mode(GTK_TREE_VIEW(tree), TRUE);
+	gtk_tree_view_set_search_column(GTK_TREE_VIEW(pl3_fb_dir_tree), PL3_FB_NAME);
 	/* set the search column */
 	gtk_tree_view_set_search_column(GTK_TREE_VIEW(tree), PL3_FB_NAME);
 
@@ -207,23 +219,22 @@ static void pl3_file_browser_init(void)
 	gtk_box_pack_start(GTK_BOX(vbox), pl3_fb_sw, TRUE, TRUE,0);
 	gtk_widget_show_all(pl3_fb_sw);	
 
-        /* Warning box for when there is no music */
-        pl3_fb_warning_box = gtk_label_new("");
-        gtk_label_set_markup(GTK_LABEL(pl3_fb_warning_box), 
-	_("It seems you have no music in your database.\n"
-	  "To add music, copy the music to your <i>music_directory</i> as specified in your mpd config file.\n"
-          "Then update the database. (Server->Update Database)"));
-        gtk_misc_set_alignment(GTK_MISC(pl3_fb_warning_box), 0.0,0.0);
-        gtk_misc_set_padding(GTK_MISC(pl3_fb_warning_box), 12,12);
+    /* Warning box for when there is no music */
+    pl3_fb_warning_box = gtk_label_new("");
+    gtk_label_set_markup(GTK_LABEL(pl3_fb_warning_box), 
+            _("It seems you have no music in your database.\n"
+                "To add music, copy the music to your <i>music_directory</i> as specified in your mpd config file.\n"
+                "Then update the database. (Server->Update Database)"));
+    gtk_misc_set_alignment(GTK_MISC(pl3_fb_warning_box), 0.0,0.0);
+    gtk_misc_set_padding(GTK_MISC(pl3_fb_warning_box), 12,12);
 	gtk_widget_set_no_show_all(pl3_fb_warning_box, TRUE);
 	gtk_box_pack_end(GTK_BOX(vbox), pl3_fb_warning_box, FALSE, TRUE,0);
 
     gtk_paned_add2(GTK_PANED(pl3_fb_vbox), vbox);
 	/* set initial state */
-	debug_printf(DEBUG_INFO,"initialized current playlist treeview\n");
     gtk_widget_show(vbox);
     gtk_widget_show(pl3_fb_vbox);
-	g_object_ref(G_OBJECT(pl3_fb_vbox));
+	g_object_ref_sink(G_OBJECT(pl3_fb_vbox));
 }
 
 static void pl3_file_browser_add_folder(void)
@@ -332,8 +343,8 @@ static void pl3_file_browser_add(GtkWidget *cat_tree)
 	playlist3_insert_browser(&iter, pos);
 	gtk_list_store_set(GTK_LIST_STORE(pl3_tree), &iter, 
 			PL3_CAT_TYPE, file_browser_plug.id,
-			PL3_CAT_TITLE, _("File Browser"),
-			PL3_CAT_ICON_ID, "gtk-open",
+			PL3_CAT_TITLE, _("Database"),
+			PL3_CAT_ICON_ID, "gmpc-database",
 			-1);
 	/* add fantom child for lazy tree */
 
@@ -370,7 +381,9 @@ static int directory_sort_func(gpointer ppaa, gpointer ppbb, gpointer data)
     MpdData_real *a = *(MpdData_real **)ppaa;
     MpdData_real *b = *(MpdData_real **)ppbb;
     int val = 0;
-    if(a->type == MPD_DATA_TYPE_DIRECTORY & b->type == MPD_DATA_TYPE_DIRECTORY)
+    if(!(a && b)) return val;
+    if(a->type != b->type) return a->type - b->type;
+    if((a->type == MPD_DATA_TYPE_DIRECTORY) && (b->type == MPD_DATA_TYPE_DIRECTORY))
     {
         if(a->directory && b->directory) {
             gchar *sa,*sb;
@@ -383,6 +396,12 @@ static int directory_sort_func(gpointer ppaa, gpointer ppbb, gpointer data)
     }
     return val;
 }
+/* Reverse sort function, needed for prepending instead of appending */
+static int directory_sort_func_inv(gpointer a, gpointer b, gpointer d)
+{
+  return - directory_sort_func (a, b, d);
+}
+
 static void pl3_file_browser_reupdate_folder(GtkTreeIter *iter)
 {
 	MpdData *data = NULL;
@@ -394,9 +413,8 @@ static void pl3_file_browser_reupdate_folder(GtkTreeIter *iter)
 	{
 		GtkTreeIter child, child2,child3;
 		data = mpd_database_get_directory(connection,path);
-
-        data = misc_sort_mpddata(data,(GCompareDataFunc)directory_sort_func,NULL); 
-        g_free(path);
+		data = misc_sort_mpddata(data,(GCompareDataFunc)directory_sort_func,NULL); 
+		g_free(path);
 		if(gtk_tree_model_iter_children(model, &child, iter))
 		{
 			gchar *test_path = NULL;
@@ -485,7 +503,7 @@ static void pl3_file_browser_reupdate(void)
 
 		GtkTreeModel *model = GTK_TREE_MODEL(pl3_fb_dir_store); 
 		
-		if(mpd_stats_get_total_songs(connection) == 0)
+		if(mpd_stats_get_total_songs(connection) == 0 && !mpd_status_db_is_updating(connection))
 		{
                     gtk_widget_show(pl3_fb_warning_box);
 		}else{
@@ -528,10 +546,8 @@ static void pl3_file_browser_view_folder(GtkTreeSelection *selection, gpointer u
 	if(strcmp("media-playlist",icon))
 	{
 		data = mpd_database_get_directory(connection, path);
-        data = misc_sort_mpddata(data,(GCompareDataFunc)directory_sort_func,NULL); 
 	}
 	else{
-		debug_printf(DEBUG_INFO,"View Playlist\n");
 		data = mpd_database_get_playlist_content(connection, path);
 	}
 	/* Check, and set the up arrow in the model */
@@ -560,43 +576,54 @@ static void pl3_file_browser_collapse_row(GtkTreeView *tree, GtkTreeIter *iter,
     /* add phantom child */
     gtk_tree_store_append(pl3_fb_dir_store, &child, iter);
 }
-
 static void pl3_file_browser_fill_tree(GtkWidget *tree,GtkTreeIter *iter, GtkTreePath *tpath, gpointer user_data)
 {
     char *path;
     MpdData *data = NULL;
-    GtkTreeIter child,child2;
+    GtkTreeIter child,child2, dummy;
     gboolean open;
     gtk_tree_model_get(GTK_TREE_MODEL(pl3_fb_dir_store),iter, PL3_FB_PATH, &path, PL3_FB_OPEN, &open,-1);
     gtk_tree_store_set(pl3_fb_dir_store, iter, PL3_FB_OPEN, TRUE, -1);
     if(open == FALSE)
     {
+        GTimer *tim = g_timer_new();
         data = mpd_database_get_directory(connection, path);
-        data = misc_sort_mpddata(data,(GCompareDataFunc)directory_sort_func,NULL); 
-        while (data != NULL)
-        {
-            if (data->type == MPD_DATA_TYPE_DIRECTORY)
-            {
-                gchar *basename =
-                    g_path_get_basename (data->directory);
-                gtk_tree_store_append (pl3_fb_dir_store, &child, iter);
-                gtk_tree_store_set (pl3_fb_dir_store, &child,
-                        PL3_FB_ICON, "gtk-open",
-                        PL3_FB_NAME, basename,
-                        PL3_FB_PATH, data->directory,
-                        PL3_FB_OPEN, FALSE,
-                        -1);
-                gtk_tree_store_append(pl3_fb_dir_store, &child2, &child);
+        /* Do a reverse sort, because adding it to the gtk view by prepending is faster
+         * then appending */
+        data = misc_sort_mpddata(data,(GCompareDataFunc)directory_sort_func_inv,NULL);
+        g_log(LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Elapsed time sorting before adding: %f\n",
+                g_timer_elapsed(tim, NULL));
 
-                q_free (basename);
-            }
-            data = mpd_data_get_next(data);
-        }
-        if(gtk_tree_model_iter_children(GTK_TREE_MODEL(pl3_fb_dir_store), &child, iter))
+        if(gtk_tree_model_iter_children(GTK_TREE_MODEL(pl3_fb_dir_store), &dummy, iter))
         {
-            gtk_tree_store_remove(pl3_fb_dir_store, &child);
+            while (data != NULL)
+            {
+                if (data->type == MPD_DATA_TYPE_DIRECTORY)
+                {
+                    gchar *basename = g_path_get_basename (data->directory);
+                    gtk_tree_store_prepend(pl3_fb_dir_store, &child, iter);
+                    gtk_tree_store_set (pl3_fb_dir_store, &child,
+                            PL3_FB_ICON, "gtk-open",
+                            PL3_FB_NAME, basename,
+                            PL3_FB_PATH, data->directory,
+                            PL3_FB_OPEN, FALSE,
+                            -1);
+                    gtk_tree_store_append(pl3_fb_dir_store, &child2, &child);
+
+                    q_free (basename);
+                }
+                data = mpd_data_get_next(data);
+            }
+
+            gtk_tree_store_remove(pl3_fb_dir_store, &dummy);
         }
+
+        g_log(LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Elapsed time sorting after adding: %f\n",
+                g_timer_elapsed(tim, NULL));
+        g_timer_destroy(tim);
+
     }
+
     q_free(path);
 }
 
@@ -624,8 +651,8 @@ static int pl3_file_browser_cat_popup(GtkWidget *wid, GdkEventButton *event, gpo
         g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(pl3_file_browser_replace_folder), NULL);
 
         {
-            GtkTreeView *tree = GTK_TREE_VIEW(pl3_fb_dir_tree); 
-            GtkTreeModel *model = (GtkTreeModel *)pl3_fb_dir_store; 
+            GtkTreeView *tree = GTK_TREE_VIEW(pl3_fb_dir_tree);
+            GtkTreeModel *model = (GtkTreeModel *)pl3_fb_dir_store;
             GtkTreeSelection *selection  = gtk_tree_view_get_selection(tree);
             GtkTreeIter iter;
             if(gtk_tree_selection_get_selected(selection, &model, &iter))
@@ -1190,9 +1217,9 @@ static int pl3_file_browser_add_go_menu(GtkWidget *menu)
     if(!pl3_file_browser_get_enabled())
         return 0;
 
-    item = gtk_image_menu_item_new_with_label(_("File Browser"));
+    item = gtk_image_menu_item_new_with_label(_("Database"));
     gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), 
-            gtk_image_new_from_stock("gtk-open", GTK_ICON_SIZE_MENU));
+            gtk_image_new_from_icon_name("gmpc-database", GTK_ICON_SIZE_MENU));
     gtk_widget_add_accelerator(GTK_WIDGET(item), 
             "activate",
             gtk_menu_get_accel_group(GTK_MENU(menu)), 
@@ -1227,6 +1254,11 @@ static void pl3_file_browser_connection_changed(MpdObj *mi, int connect, gpointe
 
 static void pl3_file_browser_status_changed(MpdObj *mi,ChangedStatusType what, void *data)
 {
+    if(what&MPD_CST_UPDATING) {
+        if(pl3_fb_vbox != NULL && mpd_status_db_is_updating(connection)){
+            gtk_widget_hide(pl3_fb_warning_box);
+        }
+    }
     if(what&MPD_CST_DATABASE)
     {
         pl3_file_browser_reupdate();
@@ -1256,7 +1288,7 @@ static void pl3_file_browser_save_myself(void)
         if(path)
         {
             gint *indices = gtk_tree_path_get_indices(path);
-            debug_printf(DEBUG_INFO,"Saving myself to position: %i\n", indices[0]);
+            g_log(LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Saving myself to position: %i\n", indices[0]);
             cfg_set_single_value_as_int(config, "file-browser","position",indices[0]);
             gtk_tree_path_free(path);
         }
@@ -1315,3 +1347,49 @@ void pl3_file_browser_open_path(const gchar *path)
         }
     }
 }
+
+
+static gboolean pl3_file_browser_is_field_supported(int tag)
+{
+    if(tag == MPD_TAG_NUM_OF_ITEM_TYPES) return TRUE; 
+    return mpd_server_tag_supported(connection, tag);
+}
+
+static MpdData * pl3_file_browser_is_search(const int num_field , const gchar *search_string, GError **error)
+{
+    MpdData *data_t = NULL;
+    if(num_field == MPD_TAG_NUM_OF_ITEM_TYPES){
+        data_t = advanced_search(search_string, FALSE);
+    }else{
+        gchar ** splitted = tokenize_string(search_string);
+        int i =0;
+        gboolean found = FALSE;
+        for(i=0;splitted && splitted[i];i++) {
+            if(!found) {
+                mpd_database_search_start(connection, FALSE);
+                found = TRUE;
+            }
+            mpd_database_search_add_constraint(connection, num_field, splitted[i]);
+        }
+        if(splitted)
+            g_strfreev(splitted);
+        if(found) {
+            data_t = mpd_database_search_commit(connection);
+        }
+    }
+    return data_t;
+}
+
+void pl3_find2_ec_database(gpointer user_data, const char *param)
+{
+    pl3_find2_select_plugin_id(file_browser_plug.id);
+    pl3_find2_do_search_any(param);
+}
+
+static void pl3_file_browser_plugin_init(void)
+{
+    gmpc_easy_command_add_entry(gmpc_easy_command, 
+                _("search database"), ".*",
+                _("Search database <query>"),
+                (GmpcEasyCommandCallback *)pl3_find2_ec_database, NULL);
+}
