@@ -1,5 +1,5 @@
 /* Gnome Music Player Client (GMPC)
- * Copyright (C) 2004-2009 Qball Cow <qball@sarine.nl>
+ * Copyright (C) 2004-2010 Qball Cow <qball@sarine.nl>
  * Project homepage: http://gmpc.wikia.com/
  
  * This program is free software; you can redistribute it and/or modify
@@ -44,10 +44,10 @@ static int pl3_find2_browser_playlist_key_press(GtkWidget *, GdkEventKey *);
 static void pl3_find2_browser_add_selected(void);
 static gboolean pl3_find2_browser_button_release_event(GtkWidget *but, GdkEventButton *event);
 static void pl3_find2_browser_connection_changed(MpdObj *mi, int connect, gpointer data);
-static int pl3_find2_browser_key_press_event(GtkWidget *mw, GdkEventKey *event, int type);
 static gboolean pl3_find2_entry_key_press_event(GtkWidget *entry, GdkEventKey *event, gpointer data);
 static void pl3_find2_browser_status_changed(MpdObj *mi,ChangedStatusType what, void *data);
 
+static void pl3_find2_entry_changed(GtkWidget *entry, gpointer data);
 static void pl3_find2_save_myself(void);
 
 
@@ -70,39 +70,23 @@ static GtkWidget            *search_entry           = NULL;
 #define QUERY_ENTRY MPD_TAG_NUM_OF_ITEM_TYPES
 static void pl3_find2_fill_combo(gmpcPluginParent *plug)
 {
-	GtkTreeIter iter;
-	int i=0, max = MPD_TAG_NUM_OF_ITEM_TYPES;
-	gtk_list_store_clear(pl3_find2_combo_store);
+    GtkTreeIter iter;
+    int i=0, max = MPD_TAG_NUM_OF_ITEM_TYPES;
+    gtk_list_store_clear(pl3_find2_combo_store);
 
-    if(plug)
+    for(i=0;i< max;i++)
     {
-        for(i=0;i< max;i++)
+        if(plug == NULL || gmpc_plugin_browser_integrate_search_field_supported(plug,i))
         {
-            if(plug == NULL || gmpc_plugin_browser_integrate_search_field_supported(plug,i))
-            {
-                gtk_list_store_append(pl3_find2_combo_store, &iter);
-                gtk_list_store_set(pl3_find2_combo_store, &iter, 1, mpdTagItemKeys[i], 0,i, -1);	
-            }
+            gtk_list_store_append(pl3_find2_combo_store, &iter);
+            gtk_list_store_set(pl3_find2_combo_store, &iter, 1, mpdTagItemKeys[i], 0,i, -1);	
         }
     }
-    else 
-    {
-        for(i=0;i< MPD_TAG_NUM_OF_ITEM_TYPES;i++)
-        {
-            if(mpd_server_tag_supported(connection,i)){
-                gtk_list_store_append(pl3_find2_combo_store, &iter);
-                gtk_list_store_set(pl3_find2_combo_store, &iter, 1, mpdTagItemKeys[i], 0,i, -1);	
-            }
-        }
-
-    }
     if(plug == NULL || gmpc_plugin_browser_integrate_search_field_supported(plug,QUERY_ENTRY))
     {
         gtk_list_store_append(pl3_find2_combo_store, &iter);
         gtk_list_store_set(pl3_find2_combo_store, &iter, 1, _("Query"), 0,QUERY_ENTRY, -1);	
     }
-    
-
 }
 
 static void pl3_find2_combo_box_field_changed(GtkComboBox *cb, gpointer data)
@@ -238,8 +222,6 @@ static void pl3_find2_browser_init(void)
     gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(pl3_find2_curpl), renderer, TRUE);
     gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(pl3_find2_curpl), renderer, "text", 0, NULL);
 
-    gtk_list_store_insert_with_values(GTK_LIST_STORE(pl3_find2_curpl_model), NULL, 0, 0, _("Database"),1,1,-1);
-    gtk_list_store_insert_with_values(GTK_LIST_STORE(pl3_find2_curpl_model), NULL, 1, 0, _("Playlist"),1,2,-1);
     for(i=0;i< num_plugins;i++){
         if(gmpc_plugin_get_enabled(plugins[i])){
             if(gmpc_plugin_is_browser(plugins[i])){
@@ -296,6 +278,7 @@ static void pl3_find2_browser_init(void)
 
     g_signal_connect(G_OBJECT(search_entry), "activate",G_CALLBACK(pl3_find2_browser_search), NULL);
     g_signal_connect(G_OBJECT(search_entry), "key-press-event",G_CALLBACK(pl3_find2_entry_key_press_event), NULL);
+    g_signal_connect(G_OBJECT(search_entry), "changed",G_CALLBACK(pl3_find2_entry_changed), NULL);
     gtk_box_pack_start(GTK_BOX(hbox), search_entry, TRUE, TRUE, 0);
 
 
@@ -309,7 +292,7 @@ static void pl3_find2_browser_init(void)
     gtk_box_pack_start(GTK_BOX(pl3_find2_vbox), hbox, FALSE,TRUE,0);
 
     gtk_widget_show_all(pl3_find2_vbox);
-    g_object_ref(G_OBJECT(pl3_find2_vbox));
+    g_object_ref_sink(G_OBJECT(pl3_find2_vbox));
 }
 
 static void pl3_find2_browser_selected(GtkWidget *container)
@@ -359,49 +342,6 @@ static void pl3_find2_browser_add(GtkWidget *cat_tree)
     }
 }
 
-static MpdData * pl3_find2_browser_view(const gint num_field, const gchar *name, gint search_playlist)
-{
-    MpdData *data_t= NULL;
-    if(mpd_server_check_command_allowed(connection, "playlistsearch")== MPD_SERVER_COMMAND_ALLOWED && 
-			mpd_server_check_command_allowed(connection, "playlistfind")== MPD_SERVER_COMMAND_ALLOWED)
-    {
-        int found = 0;
-        //        if(gtk_combo_box_get_active_iter(GTK_COMBO_BOX(search_combo), &cc_iter) && name && name[0] != '\0')
-        if(num_field == QUERY_ENTRY)
-        {
-            data_t = advanced_search(name, search_playlist);
-        }else{
-            gchar ** splitted = tokenize_string(name);
-            int i =0;
-            for(i=0;splitted && splitted[i];i++)
-            {                                                          
-                if(!found)
-                {
-                    if(search_playlist)
-                        mpd_playlist_search_start(connection, FALSE);
-                    else
-                        mpd_database_search_start(connection, FALSE);
-                    found = TRUE;
-                }
-                if(search_playlist)
-                    mpd_playlist_search_add_constraint(connection, num_field, splitted[i]);
-                else
-                    mpd_database_search_add_constraint(connection, num_field, splitted[i]);
-            }
-            if(splitted)
-                g_strfreev(splitted);
-            if(found)
-            {
-                if(search_playlist)
-                    data_t = mpd_playlist_search_commit(connection);
-                else
-                    data_t = mpd_database_search_commit(connection);
-            }
-        }
-    }
-    return data_t;
-}
-
 /**
  * Search 
  */
@@ -420,25 +360,19 @@ static void pl3_find2_browser_search(void)
 
         if(gtk_combo_box_get_active_iter(GTK_COMBO_BOX(search_combo), &cc_iter))
         {
+            GError *error = NULL;
             MpdData *data_t = NULL;
-            const gchar *name = gtk_entry_get_text(GTK_ENTRY(search_entry));
+            const gchar *real_name = gtk_entry_get_text(GTK_ENTRY(search_entry));
+            gchar *name = g_strstrip(g_strdup(real_name));
             gtk_tree_model_get(GTK_TREE_MODEL(pl3_find2_combo_store),&cc_iter , 0, &num_field, -1);
-            if(type == 1) {
-                data_t = pl3_find2_browser_view(num_field,name,FALSE);
-            }
-            else if (type == 2) {
-                data_t = pl3_find2_browser_view(num_field,name,TRUE);
-            }else if (type == 0) {
-                GError *error = NULL;
-                if(plug)
-                    data_t = gmpc_plugin_browser_integrate_search(plug,num_field, name,&error); 
-                if(error){
-                    gchar *mes = g_markup_printf_escaped("<b>%s</b> %s: %s",gmpc_plugin_get_name(plug),_("Search failed"), error->message);
-                    playlist3_show_error_message(mes, ERROR_WARNING);
-                    g_free(mes);
-                    g_error_free(error);
-                    error = NULL;
-                }
+            if(plug)
+                data_t = gmpc_plugin_browser_integrate_search(plug,num_field, name,&error); 
+            if(error){
+                gchar *mes = g_markup_printf_escaped("<b>%s</b> %s: %s",gmpc_plugin_get_name(plug),_("Search failed"), error->message);
+                playlist3_show_error_message(mes, ERROR_WARNING);
+                g_free(mes);
+                g_error_free(error);
+                error = NULL;
             }
             gmpc_mpddata_model_set_mpd_data(pl3_find2_store2, data_t);
             /* update autocompletion */
@@ -460,6 +394,7 @@ static void pl3_find2_browser_search(void)
                     gtk_list_store_insert_with_values(pl3_find2_autocomplete, &iter,-1, 0,name,-1);
                 }					
             }
+            g_free(name);
         }
     }
     gtk_tree_view_set_model(GTK_TREE_VIEW(pl3_find2_tree), GTK_TREE_MODEL(pl3_find2_store2));
@@ -704,7 +639,7 @@ static gboolean pl3_find2_browser_button_release_event(GtkWidget *but, GdkEventB
     {
       GtkWidget *item;
       GtkWidget *menu = gtk_menu_new();
-
+/*
 			if(gtk_tree_selection_count_selected_rows(gtk_tree_view_get_selection(GTK_TREE_VIEW(pl3_find2_tree))) == 1)	
 			{
 				item = gtk_image_menu_item_new_from_stock(GTK_STOCK_MEDIA_PLAY,NULL);
@@ -712,7 +647,7 @@ static gboolean pl3_find2_browser_button_release_event(GtkWidget *but, GdkEventB
 				g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(pl3_find2_activate_row), pl3_find2_tree);
 			}
 
-
+*/
       item = gtk_image_menu_item_new_from_stock(GTK_STOCK_ADD,NULL);
       gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
       g_signal_connect(G_OBJECT(item), "activate",
@@ -819,42 +754,52 @@ static void pl3_find2_browser_connection_changed(MpdObj *mi, int connect, gpoint
     }
 }
 
-/**
- * Handle program-global key events
- */
-static int pl3_find2_browser_key_press_event(GtkWidget *mw, GdkEventKey *event, int type)
+void pl3_find2_select_plugin_id(int id)
 {
-    if(event->state&GDK_CONTROL_MASK && event->keyval == GDK_j )
-    {
-        GtkTreeIter iter;
-        GtkTreeModel *model;
-        pl3_find2_browser_activate();
-        model = gtk_combo_box_get_model(GTK_COMBO_BOX(search_combo));        
-        gtk_combo_box_set_active(GTK_COMBO_BOX(pl3_find2_curpl), 1);
-        if(gtk_tree_model_get_iter_first(model, &iter)){
-            int found = 1;
-            do{
-                int field;
-                gtk_tree_model_get(model, &iter,0, &field, -1); 
-                if(field == QUERY_ENTRY){
-                    gtk_combo_box_set_active_iter(GTK_COMBO_BOX(search_combo), &iter); 
-                    found = 0;
-                }
-            }while(gtk_tree_model_iter_next(model, &iter) && found);
-        }
-        gtk_widget_grab_focus(search_entry);
-        return TRUE;
+    GtkTreeIter iter;
+    GtkTreeModel *model;
+    pl3_find2_browser_activate();
+    model = gtk_combo_box_get_model(GTK_COMBO_BOX(pl3_find2_curpl));        
+    if(gtk_tree_model_get_iter_first(model, &iter)){
+        int found = 1;
+        do{
+            gmpcPluginParent *plug;
+            gtk_tree_model_get(model, &iter,2, &plug, -1); 
+            if(plug != NULL  && gmpc_plugin_get_id(plug) == id){ 
+                gtk_combo_box_set_active_iter(GTK_COMBO_BOX(pl3_find2_curpl), &iter);
+                found = 0;
+            }
+        }while(gtk_tree_model_iter_next(model, &iter) && found);
     }
+    gtk_widget_grab_focus(search_entry);
+}
 
+static guint entry_timeout = 0;
+
+static gboolean do_entry_changed_search(GtkEntry *entry)
+{
+    pl3_find2_browser_search();
+    entry_timeout = 0;
     return FALSE;
 }
 
+static void pl3_find2_entry_changed(GtkWidget *entry, gpointer data)
+{
+    if(entry_timeout > 0) {
+        g_source_remove(entry_timeout);
+        entry_timeout = 0;
+    }
+    if(cfg_get_single_value_as_int_with_default(config, "general", "search-as-you-type", 0) == 1)
+    {
+        entry_timeout = g_timeout_add(250, (GSourceFunc)do_entry_changed_search, entry);
+    }
+}
 static gboolean pl3_find2_entry_key_press_event(GtkWidget *entry, GdkEventKey *event, gpointer data)
 {
 	if(event->keyval == GDK_Escape)
 		gtk_entry_set_text(GTK_ENTRY(entry), "");
 
-	return FALSE;
+    return FALSE;
 }
 
 static void pl3_find2_browser_destroy(void)
@@ -907,7 +852,6 @@ static void pl3_find2_save_myself(void)
 		if(path)
 		{
 			gint *indices = gtk_tree_path_get_indices(path);
-			debug_printf(DEBUG_INFO,"Saving myself to position: %i\n", indices[0]);
 			cfg_set_single_value_as_int(config, "find2-browser","position",indices[0]);
 			gtk_tree_path_free(path);
 		}
@@ -915,38 +859,11 @@ static void pl3_find2_save_myself(void)
 }
 /* Easy command integration */
 
-void pl3_find2_ec_database(gpointer user_data, const char *param)
+void pl3_find2_do_search_any(const char *param)
 {
     GtkTreeIter iter;
     GtkTreeModel *model;
-    pl3_find2_browser_activate();
-
     model  = gtk_combo_box_get_model(GTK_COMBO_BOX(search_combo));
-    gtk_combo_box_set_active(GTK_COMBO_BOX(pl3_find2_curpl), 0);
-
-    if(gtk_tree_model_get_iter_first(model, &iter)){
-        int found = 1;
-        do{
-            int field;
-            gtk_tree_model_get(model, &iter,0, &field, -1); 
-            if(field == QUERY_ENTRY){
-                gtk_combo_box_set_active_iter(GTK_COMBO_BOX(search_combo), &iter); 
-                found = 0;
-            }
-        }while(gtk_tree_model_iter_next(model, &iter) && found);
-    }
-    gtk_entry_set_text(GTK_ENTRY(search_entry), param);
-    gtk_widget_activate(search_entry);
-}
-static void pl3_find2_ec_playlist(gpointer user_data, const char *param)
-{
-    GtkTreeIter iter;
-    GtkTreeModel *model;
-    pl3_find2_browser_activate();
-
-    model  = gtk_combo_box_get_model(GTK_COMBO_BOX(search_combo));
-    gtk_combo_box_set_active(GTK_COMBO_BOX(pl3_find2_curpl), 1);
-
     if(gtk_tree_model_get_iter_first(model, &iter)){
         int found = 1;
         do{
@@ -967,16 +884,6 @@ static void pl3_find2_plugin_init(void)
                 _("switch search"),"",
                 _("Switch to the search browser"),
                 (GmpcEasyCommandCallback *)pl3_find2_browser_activate, NULL); 
-
-    gmpc_easy_command_add_entry(gmpc_easy_command, 
-                _("search database"), ".*",
-                _("Search database <query>"),
-                (GmpcEasyCommandCallback *)pl3_find2_ec_database , NULL);
-    gmpc_easy_command_add_entry(gmpc_easy_command,
-                _("search playlist"), ".*",
-                _("Search playlist <query>"),
-                (GmpcEasyCommandCallback *)pl3_find2_ec_playlist, NULL);
-
 }
 /**
  * Plugin structure
@@ -986,7 +893,6 @@ gmpcPlBrowserPlugin find2_browser_gbp = {
 	.selected = pl3_find2_browser_selected,
 	.unselected = pl3_find2_browser_unselected,
 	.add_go_menu = pl3_find2_browser_add_go_menu,
-	.key_press_event = pl3_find2_browser_key_press_event
 };
 
 gmpcPlugin find2_browser_plug = {
