@@ -1,5 +1,5 @@
 /* Gnome Music Player Client (GMPC)
- * Copyright (C) 2004-2009 Qball Cow <qball@sarine.nl>
+ * Copyright (C) 2004-2010 Qball Cow <qball@sarine.nl>
  * Project homepage: http://gmpc.wikia.com/
  
  * This program is free software; you can redistribute it and/or modify
@@ -23,4 +23,6 @@
 extern gmpcPlugin find2_browser_plug;
 
 void pl3_find2_ec_database(gpointer user_data, const char *param);
+void pl3_find2_select_plugin_id(int id);
+void pl3_find2_do_search_any(const char *param);
 #endif
