@@ -1,5 +1,5 @@
 /* Gnome Music Player Client (GMPC)
- * Copyright (C) 2004-2009 Qball Cow <qball@sarine.nl>
+ * Copyright (C) 2004-2010 Qball Cow <qball@sarine.nl>
  * Project homepage: http://gmpc.wikia.com/
  
  * This program is free software; you can redistribute it and/or modify
@@ -27,8 +27,9 @@
 #include "gmpc-mpddata-model.h"
 #include "gmpc-mpddata-treeview.h"
 #include "gmpc-mpddata-model-sort.h"
-#include "gmpc-paned-size-group.h"
+#include "gmpc-extras.h"
 #include "browsers/playlist3-playlist-editor.h"
+#include "Widgets/mpd-async-request.h"
 
 #define DEFAULT_MARKUP_BROWSER 	"[%name%: &[%artist% - ]%title%]|%name%|[%artist% - ]%title%|%shortfile%|"
 
@@ -159,7 +160,17 @@ void playlist_editor_browser_add(GtkWidget *cat_tree)
 
 }
 
-
+static MpdData *__playlist_editor_async_function(MpdObj *mi, gpointer function_data)
+{
+    MpdData* data = mpd_database_get_playlist_content(mi, (gchar *)function_data); 
+    g_free(function_data);
+    return data;
+}
+static void __playlist_editor_async_callback(MpdData *data, gpointer callback_data)
+{
+    gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(playlist_editor_list_store), data);		
+    gtk_widget_set_sensitive(playlist_editor_browser, TRUE);
+}
 
 static void playlist_editor_browser_playlist_editor_selected(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer userdata)
 {
@@ -167,10 +178,10 @@ static void playlist_editor_browser_playlist_editor_selected(GtkTreeModel *model
     MpdData *data ;
     gtk_tree_model_get(GTK_TREE_MODEL(playlist_editor_store), iter, PL_NAME, &pl_path, -1);
     if(pl_path){
-        data = mpd_database_get_playlist_content(connection, pl_path); 
-        gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(playlist_editor_list_store), data);		
+        gtk_widget_set_sensitive(playlist_editor_browser, FALSE);
+        gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(playlist_editor_list_store), NULL);		
         gmpc_mpddata_model_sort_set_playlist(GMPC_MPDDATA_MODEL_SORT(playlist_editor_list_store), pl_path);
-
+        mpd_async_request(__playlist_editor_async_callback,NULL, __playlist_editor_async_function, g_strdup(pl_path));
         if(old_playlist) g_free(old_playlist);
         old_playlist =pl_path;
     }
@@ -388,7 +399,6 @@ static void playlist_editor_list_delete_songs(GtkButton *button, GtkTreeView *tr
 				mpd_database_playlist_list_delete(connection, pl_path,pos[0]);
 			}
 		}
-        debug_printf(DEBUG_INFO,"ping\n");
 		g_list_foreach (list,(GFunc) gtk_tree_path_free, NULL);
 		g_list_free (list);
 
@@ -927,7 +937,7 @@ static void playlist_editor_browser_init(void)
 
     g_signal_connect(G_OBJECT(tree), "button-release-event", G_CALLBACK(playlist_editor_key_released), NULL);
     g_signal_connect(G_OBJECT(tree), "key-press-event", G_CALLBACK(playlist_editor_key_pressed), NULL);
-    g_object_ref(playlist_editor_browser);
+    g_object_ref_sink(playlist_editor_browser);
 
     gtk_widget_show_all(playlist_editor_browser);
 }
@@ -1090,7 +1100,6 @@ static void playlist_editor_save_myself(void)
         if(path)
         {
             gint *indices = gtk_tree_path_get_indices(path);
-            debug_printf(DEBUG_INFO,"Saving myself to position: %i\n", indices[0]);
             cfg_set_single_value_as_int(config, "playlist-plugin","position",indices[0]);
             gtk_tree_path_free(path);
         }
