@@ -1,5 +1,5 @@
 /* Gnome Music Player Client (GMPC)
- * Copyright (C) 2004-2009 Qball Cow <qball@sarine.nl>
+ * Copyright (C) 2004-2010 Qball Cow <qball@sarine.nl>
  * Project homepage: http://gmpc.wikia.com/
  
  * This program is free software; you can redistribute it and/or modify
@@ -30,7 +30,6 @@
 #include <string.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
-#include <libmpd/debug_printf.h>
 #include "main.h"
 #include "misc.h"
 #include "gmpc-mpddata-treeview.h"
@@ -41,9 +40,9 @@
 #include <libmpd/libmpd-internal.h>
 #include "playlist3.h"
 #include "playlist3-playlist-editor.h"
-#include "gmpc-paned-size-group.h"
-#include "gmpc-mpddata-treeview-tooltip.h"
+#include "gmpc-extras.h"
 
+#define LOG_DOMAIN "TagBrowser"
 /**
  * dirty hack to workaround single parameter for now 
  */
@@ -114,6 +113,8 @@ gmpcPlugin tag2_plug = {
 	.mpd_connection_changed = tag2_connection_changed,
 	.mpd_status_changed = tag2_status_changed,
 };
+
+
 /** Little hack to work around gmpc's limitations */
 static GList *tag2_ht = NULL;
 /** This stucture contains all the needed data for a browser
@@ -336,7 +337,6 @@ static void tag2_browser_add_selected(GtkWidget *item, tag_element *te)
  * Redirect to metadata browser 
  */
 
-#include "vala/gmpc-metadata-browser2.h"
 extern GmpcMetadataBrowser *metadata_browser;
 static void tag2_browser_header_information(GtkWidget *item, tag_element *te)
 {
@@ -622,6 +622,7 @@ static void tag2_changed(GtkTreeSelection *sel2, tag_element *te)
              * This will make sure, selected rows that are matched, don't get de-selected 
              */
             sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(te_i->tree));
+
             gmpc_mpddata_model_set_mpd_data_slow(GMPC_MPDDATA_MODEL(te_i->model), data);
             /* this make sure the selected row is centered in the middle of the treeview.
              * Otherwise the user could have the tedious job of finding it again
@@ -769,7 +770,7 @@ static void tag2_column_header_menu_item_clicked(GtkCheckMenuItem *item, tag_ele
             {
                 mpd_database_search_field_start(connection, te->type);
                 data = mpd_database_search_commit(connection);
-                gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(te->model), data);
+                gmpc_mpddata_model_set_mpd_data_slow(GMPC_MPDDATA_MODEL(te->model), data);
             }else {
                 gmpc_mpddata_model_set_mpd_data(GMPC_MPDDATA_MODEL(te->model), NULL);
             }
@@ -932,6 +933,7 @@ static void tag2_songlist_add_tag(tag_browser *browser,const gchar *name, int ty
     te->tool    = gmpc_mpd_data_treeview_tooltip_new(GTK_TREE_VIEW(te->tree), 0);
 	te->browser = browser;
 
+    gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(te->tree), TRUE);
     if(te->type == MPD_TAG_ITEM_ARTIST ||te->type == MPD_TAG_ITEM_ALBUM_ARTIST) {
         te->tool->mtype = META_ARTIST_ART;
     } else if(te->type == MPD_TAG_ITEM_ALBUM ) {
@@ -1237,7 +1239,7 @@ static void tag2_init_browser(tag_browser *browser) {
 	/* Create an extra reference to the paned window containing everything, this way
 	 * I can add/remove it from gmpc's container withouth it being destroyed by gtk
 	 */
-	g_object_ref(browser->tag2_vbox);
+	g_object_ref_sink(browser->tag2_vbox);
 	/* show everything */
 	gtk_widget_show_all(browser->tag2_vbox);
 
@@ -1851,7 +1853,7 @@ static void tag2_save_myself(void)
 			{
 				gint *indices = gtk_tree_path_get_indices(path);
 				gchar *group = g_strdup_printf("tag2-plugin:%s",tb->key);
-				debug_printf(DEBUG_INFO,"Saving myself to position: %i\n", indices[0]);
+				g_log(LOG_DOMAIN, G_LOG_LEVEL_DEBUG,"Saving myself to position: %i\n", indices[0]);
 				cfg_set_single_value_as_int(config, group,"position",indices[0]);
 				gtk_tree_path_free(path);
 				g_free(group);
