@@ -1,5 +1,5 @@
 /* gmpc-serverstats (GMPC plugin)
- * Copyright (C) 2006-2009 Qball Cow <qball@sarine.nl>
+ * Copyright (C) 2006-2010 Qball Cow <qball@sarine.nl>
  * Project homepage: http://gmpc.wikia.com/
  
  * This program is free software; you can redistribute it and/or modify
@@ -438,7 +438,7 @@ static void serverstats_init(void)
     gtk_widget_show(serverstats_vbox);	
     gtk_widget_show(serverstats_event);	
     gtk_widget_show(serverstats_sw);	
-    g_object_ref(serverstats_sw);
+    g_object_ref_sink(serverstats_sw);
 }
 
 static guint timeout_source = 0;
@@ -485,7 +485,6 @@ static void serverinformation_popup_close(GtkWidget *dialog, gint response_id, g
     /* destroy dialog */
     gtk_widget_destroy(dialog);
     
-    debug_printf(DEBUG_INFO,"Close dialog: %i %i",width,height);
 }
 
 static void serverstats_connection_changed(MpdObj *mi, int connect,void *usedata)
