@@ -1,5 +1,5 @@
 /* Gnome Music Player Client (GMPC)
- * Copyright (C) 2004-2009 Qball Cow <qball@sarine.nl>
+ * Copyright (C) 2004-2010 Qball Cow <qball@sarine.nl>
  * Project homepage: http://gmpc.wikia.com/
  
  * This program is free software; you can redistribute it and/or modify
@@ -45,19 +45,12 @@
 #include "browsers/playlist3-tag2-browser.h"
 #include "browsers/playlist3-current-playlist-browser.h"
 
-/* vala */
-#include "vala/gmpc-easy-command.h"
-#include "vala/gmpc-test-plugin.h"
-#include "vala/gmpc-metadata-browser2.h"
-#include "vala/gmpc-metadata-prefetcher.h"
-#include "vala/gmpc-database-update-tracker.h"
-#include "vala/gmpc-paned-size-group.h"
-#include "vala/gmpc-nowplaying2.h"
-
 #include "gmpc-mpddata-model-playlist.h"
 #include "metadata-cache.h"
 #include "bug-information.h"
 
+#include "Widgets/pixbuf-cache.h"
+#include <libmpd/debug_printf.h>
 
 #define LOG_DOMAIN "Gmpc"
 /**
@@ -112,11 +105,6 @@ static void gmpc_status_changed_callback_real(GmpcConnection * gmpcconn,
 /**
  * Define some local functions
  */
-/**
- * Error dialog
- */
-static GtkWidget *error_dialog = NULL;
-static GtkListStore *error_list_store = NULL;
 
 /** handle connection changed */
 static void connection_changed(MpdObj * mi, int connect, gpointer data);
@@ -155,8 +143,8 @@ void send_password(void);
  * Set paths
  */
 static void create_gmpc_paths(void);
-
-void print_version(void);
+static void move_old_gmpc_data(void);
+static void print_version(void);
 
 #ifndef WIN32
 #include "bacon/bacon-message-connection.h"
@@ -216,6 +204,9 @@ static void bacon_on_message_received(const char *message, gpointer data)
          */
 		else if (strncmp(message, "STREAM ", 7) == 0) {
 			url_start_real(&message[7]);
+		}
+		else if (strncmp(message, "EASYCOMMAND", strlen("EASYCOMMAND")) == 0) {
+			gmpc_easy_command_do_query(gmpc_easy_command, (&message[strlen("EASYCOMMAND")]));
 		} else {
 			create_playlist3();
 		}
@@ -228,6 +219,17 @@ static void bacon_on_message_received(const char *message, gpointer data)
 #endif
 
 static GLogLevelFlags global_log_level = G_LOG_LEVEL_MESSAGE;
+
+
+
+static void xml_error_func(void * ctx, const char * msg,...)
+{
+	va_list ap;
+	va_start(ap, msg);
+	g_logv("LibXML", G_LOG_LEVEL_DEBUG, msg,ap);
+	va_end(ap);
+}
+
 static void gmpc_log_func(const gchar *log_domain, GLogLevelFlags log_level, const gchar *message, gpointer user_data)
 {
 	if(log_level <= global_log_level)
@@ -252,6 +254,7 @@ static gboolean hide_on_start(void)
 }
 int main(int argc, char **argv)
 {
+	static xmlGenericErrorFunc handler = (xmlGenericErrorFunc)xml_error_func;
 	int i;
 
 #ifdef WIN32
@@ -274,6 +277,7 @@ int main(int argc, char **argv)
 	gboolean		fullscreen				= FALSE;
 	gchar			*config_path			= NULL;
 	gint			debug_level				= -1;
+	gchar			*profile_name			= NULL;
 	
 	GOptionEntry entries[] = 
 	{
@@ -301,7 +305,8 @@ int main(int argc, char **argv)
 			&show_bug_information, N_("Show bug information dialog"),				NULL},
 		{ "log-filter",		'f', 0,G_OPTION_ARG_CALLBACK, 
 			set_log_filter, N_("Shows all output from a certain log domain"),		"<Log domain>"},
-			
+		{ "profile",		'p', 0,G_OPTION_ARG_STRING, 
+			&profile_name , N_("Select a profile"),								"<Profile Name>"},
 
 		{NULL}
 	};
@@ -398,6 +403,7 @@ int main(int argc, char **argv)
      */
 	xmlInitParser();
 
+	initGenericErrorDefaultFunc(&handler);
 	/**
      * Check if threading is supported, if so, start it.
      * Don't fail here, stuff like can cause that it is allready initialized.
@@ -455,6 +461,7 @@ int main(int argc, char **argv)
      * Call create_gmpc_paths();
      * This function checks if the path needed path are available, if not, create them
      */
+	move_old_gmpc_data();
 	create_gmpc_paths();
 	TEC("Check version and create paths");
 
@@ -495,30 +502,30 @@ int main(int argc, char **argv)
 
 	/** test if config opened correct  */
 	if (config == NULL) {
-	/**
-         * Show gtk error message and quit
-         */
+		/**
+		 * Show gtk error message and quit
+		 */
 		g_log(LOG_DOMAIN, G_LOG_LEVEL_ERROR, "Failed to save/load configuration:\n%s\n", url);
-		show_error_message(_("Failed to load the configuration system."), TRUE);
+		show_error_message(_("Failed to load the configuration system."));
 		/* this is an error so bail out correctly */
 		abort();
 	}
 	TEC("Opening config file: %s", url);
 	/**
-     * cleanup
-     */
+	 * cleanup
+	 */
 	q_free(url);
 
 	/**
-     * If requested, output debug info to file
-     */
+	 * If requested, output debug info to file
+	 */
 	if (cfg_get_single_value_as_int_with_default(config, "Default", "Debug-log", FALSE)) {
 		url = gmpc_get_user_path("debug-info.log");
 		if (url) {
 			FILE *fp = g_fopen(url, "a");
 			if (!fp) {
 				g_log(LOG_DOMAIN, G_LOG_LEVEL_ERROR, "Failed to open debug-log file: \"%s\"\n", url);
-				show_error_message(_("Failed to load debug-log file."), TRUE);
+				show_error_message(_("Failed to load debug-log file."));
 				abort();
 			}
 			/* Set the output */
@@ -622,31 +629,45 @@ int main(int argc, char **argv)
 	/** Signals */
 
 	gmpc_profiles = gmpc_profiles_new();
+	if(profile_name) {
+		GList *iter, *items = gmpc_profiles_get_profiles_ids(gmpc_profiles);
+		for(iter = g_list_first(items); iter; iter = g_list_next(iter))
+		{
+			if(g_utf8_collate(profile_name, gmpc_profiles_get_name(gmpc_profiles, (const gchar *)iter->data)) == 0)
+			{
+				connection_set_current_profile((const gchar *)iter->data);
+				break;
+			}
+		}
+		g_list_foreach(items,(GFunc) g_free, NULL);
+		g_list_free(items);
+
+	}
 	TEC("Setting up gmpc idle,signals and profiles");
 	/**
-     * Initialize the new metadata subsystem.
-     * (Will spawn a new thread, so have to be after the init threading
-     */
+	 * Initialize the new metadata subsystem.
+	 */
 	meta_data_init();
+	pixbuf_cache_create();
 
 	TEC("Initializing metadata system");
 
 	/**
-     * stock icons
-     */
+	 * stock icons
+	 */
 	g_log(LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Loading stock icons");
 	init_stock_icons();
 	TEC("Init stock icons");
 	/**
-     * Create connection object
-     */
+	 * Create connection object
+	 */
 	connection = mpd_new_default();
 	if (connection == NULL) {
 	/**
          * if failed, print error message
          */
 		g_log(LOG_DOMAIN, G_LOG_LEVEL_ERROR, "Failed to create connection object\n");
-		show_error_message(_("Failed to setup libmpd"), TRUE);
+		show_error_message(_("Failed to setup libmpd"));
 		abort();
 	}
 	TEC("Setting up mpd connection object");
@@ -678,14 +699,15 @@ int main(int argc, char **argv)
 	pl3_messages = playlist3_message_plugin_new();
 
 
-    playlist = (GtkTreeModel *)gmpc_mpddata_model_playlist_new(gmpcconn,connection);
-	g_object_ref_sink(playlist);
-    gmpc_mpddata_model_disable_image(GMPC_MPDDATA_MODEL(playlist));
-	/** current playlist */
-	plugin_add_new((GmpcPluginBase *)play_queue_plugin_new("current-pl"), 0,NULL);
+	playlist = (GtkTreeModel *)gmpc_mpddata_model_playlist_new(gmpcconn,connection);
+	gmpc_mpddata_model_disable_image(GMPC_MPDDATA_MODEL(playlist));
 
 	/** file browser */
 	plugin_add(&file_browser_plug, 0, NULL);
+	/** current playlist */
+	plugin_add_new((GmpcPluginBase *)play_queue_plugin_new("current-pl"), 0,NULL);
+	plugin_add_new((GmpcPluginBase *)gmpc_provider_music_tree_new(),0,NULL);
+
 	/** Find Browser */
 	plugin_add(&find2_browser_plug, 0, NULL);
 	/* this shows the connection preferences */
@@ -714,7 +736,6 @@ int main(int argc, char **argv)
 	plugin_add_new(GMPC_PLUGIN_BASE(gmpc_test_plugin_new()), 0, NULL);
 	metadata_browser = gmpc_metadata_browser_new();
 	plugin_add_new(GMPC_PLUGIN_BASE(metadata_browser), 0, NULL);
-//	plugin_add_new(GMPC_PLUGIN_BASE(gmpc_now_playing_new()), 0, NULL);
 	plugin_add_new(GMPC_PLUGIN_BASE(gmpc_plugin_metadata_prefetcher_new()), 0,NULL);
 	plugin_add_new(GMPC_PLUGIN_BASE(gmpc_plugin_database_update_tracker_new()), 0,NULL);
 	plugin_add_new(GMPC_PLUGIN_BASE(gmpc_plugin_mockup_new()), 0,NULL);
@@ -737,7 +758,7 @@ int main(int argc, char **argv)
 		plugin_load_dir(url);
 		q_free(url);
 #endif
-
+		/* Load plugin from $PLUGIN_DIR if set */
 		if(g_getenv("PLUGIN_DIR") != NULL) {
 			gchar *path = g_build_filename(g_getenv("PLUGIN_DIR"),NULL);
 			if (path && g_file_test(path, G_FILE_TEST_IS_DIR)) {
@@ -774,6 +795,24 @@ int main(int argc, char **argv)
     /* Set window debug, this is used for developers to visualize redraws */
 	g_log(LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "Create main window\n");
 	gdk_window_set_debug_updates(do_debug_updates);
+
+	if( (revision != NULL && revision[0] != '\0') && cfg_get_single_value_as_int_with_default(config, "Default", "help-question", 0) < 2 && cfg_get_single_value_as_int_with_default(config, "Default", "first-run", 1)) 
+	{
+		GtkWidget * dialog = gtk_message_dialog_new_with_markup(NULL, 
+				GTK_DIALOG_MODAL,
+				GTK_MESSAGE_QUESTION,GTK_BUTTONS_CLOSE,
+				_("The GMPC  team is looking for help maintaining and developing GMPC\n"
+				  "Help is needed with the following:\n"
+				  "* Maintaining and updating the website.\n"
+				  "* User support\n"
+				  "* Maintaining plugins\n"
+				  "* With maintaining GMPC itself\n"
+				  "If you are interested in helping out, please join our irc channel (#gmpc on irc.freenode.net)\n"
+				  "\nThanks,\n<i>Qball Cow</i>"));
+		gtk_dialog_run(GTK_DIALOG(dialog));
+		gtk_widget_destroy(dialog);
+		cfg_set_single_value_as_int(config, "Default", "help-question", 2);
+	}
 	/**
      * Create the main window
      */
@@ -807,11 +846,11 @@ int main(int argc, char **argv)
 	 * get the status every 1/2 second should be enough, but it's configurable.
 	 */
 	g_timeout_add(cfg_get_single_value_as_int_with_default(config,
-														   "connection",
-														   "mpd-update-speed",
-														   500), (GSourceFunc) update_mpd_status, NULL);
+				"connection",
+				"mpd-update-speed",
+				500), (GSourceFunc) update_mpd_status, NULL);
 	/**
-     * create the autoconnect timeout, if autoconnect enable, it will check every 5 seconds
+	 * create the autoconnect timeout, if autoconnect enable, it will check every 5 seconds
      * if you are still connected, and reconnects you if not.
      */
 	autoconnect_timeout = g_timeout_add_seconds(5, (GSourceFunc) autoconnect_callback, NULL);
@@ -899,7 +938,6 @@ int main(int argc, char **argv)
 		gmpc_plugin_destroy(plugins[i]);
 	}
 
-	//playlist3_message_destroy();
 	playlist3_destroy();
 
 	g_object_unref(playlist);
@@ -916,6 +954,8 @@ int main(int argc, char **argv)
 	g_object_unref(gmpc_profiles);
 	g_object_unref(gmpcconn);
 
+
+	pixbuf_cache_destroy();
 	/**
 	 * This now gets destroyed with the plugins
 	 */
@@ -925,6 +965,7 @@ int main(int argc, char **argv)
      */
 	mpd_free(connection);
 
+	initGenericErrorDefaultFunc((xmlGenericErrorFunc *)NULL);
 	xmlCleanupParser();
 	/* cleanup */
 	gmpc_mpddata_treeview_cleanup();
@@ -1028,6 +1069,7 @@ void GmpcStatusChangedCallback(MpdObj * mi, ChangedStatusType what, void *userda
 static void gmpc_status_changed_callback_real(GmpcConnection * conn, MpdObj * mi, ChangedStatusType what, gpointer data)
 {
 	int i;
+	/* When permission changes, update the advanced search regex */
 	if(what&MPD_CST_PERMISSION){
 		advanced_search_update_taglist();
 	}
@@ -1037,7 +1079,6 @@ static void gmpc_status_changed_callback_real(GmpcConnection * conn, MpdObj * mi
 	for (i = 0; i < num_plugins; i++) {
 		gmpc_plugin_status_changed(plugins[i], mi, what);
 	}
-
 }
 
 /*******************************
@@ -1050,10 +1091,7 @@ static void password_dialog_response(GtkWidget * dialog, gint response, gpointer
 	gchar *path;
 	switch (response) {
 	case 0:
-	{
-
 		return;
-	}
 	case GTK_RESPONSE_OK:
 		{
 			path = (char *)
@@ -1074,7 +1112,7 @@ static void password_dialog_response(GtkWidget * dialog, gint response, gpointer
 		break;
 	}
 	gtk_widget_destroy((GtkWidget *)
-					   gtk_builder_get_object(xml_password_window, "password-dialog"));
+			gtk_builder_get_object(xml_password_window, "password-dialog"));
 	g_object_unref(xml_password_window);
 	xml_password_window = NULL;
 }
@@ -1086,25 +1124,27 @@ static void password_dialog(int failed)
 	if (xml_password_window)
 		return;
 	path = gmpc_get_full_glade_path("password-dialog.ui");
-	xml_password_window = gtk_builder_new();	//glade_xml_new(path, "password-dialog",NULL);
+	xml_password_window = gtk_builder_new();
 	gtk_builder_add_from_file(xml_password_window, path, NULL);
 	gtk_window_set_transient_for(GTK_WINDOW
-								 (gtk_builder_get_object(xml_password_window, "password-dialog")), GTK_WINDOW(pl3_win));
+			(gtk_builder_get_object(xml_password_window, "password-dialog")), GTK_WINDOW(pl3_win));
 	q_free(path);
 	if (!xml_password_window)
 		return;
 	if (failed) {
-		path = g_strdup_printf(_("Failed to set password on: '%s'\nPlease try again"), mpd_get_hostname(connection));
+		path = g_strdup_printf(_("Failed to set password on: '%s'\nPlease try again"),
+				mpd_get_hostname(connection));
 	} else {
-		path = g_strdup_printf(_("Please enter your password for: '%s'"), mpd_get_hostname(connection));
+		path = g_strdup_printf(_("Please enter your password for: '%s'"),
+				mpd_get_hostname(connection));
 	}
 	gtk_label_set_text(GTK_LABEL(gtk_builder_get_object(xml_password_window, "pass_label")), path);
 	q_free(path);
 
 	g_signal_connect(G_OBJECT
-					 (gtk_builder_get_object
-					  (xml_password_window, "password-dialog")), "response",
-					 G_CALLBACK(password_dialog_response), xml_password_window);
+			(gtk_builder_get_object
+			 (xml_password_window, "password-dialog")), "response",
+			G_CALLBACK(password_dialog_response), xml_password_window);
 }
 
 void send_password(void)
@@ -1135,7 +1175,7 @@ static int error_callback(MpdObj * mi, int error_id, char *error_msg, gpointer d
 		if (setup_assistant_is_running()
 			&& (error_id == MPD_ACK_ERROR_PERMISSION || error_id == MPD_ACK_ERROR_PASSWORD)) {
 			gchar *str = g_markup_printf_escaped("<b>%s</b>",
-												 _("Insufficient permission to connect to mpd. Check password"));
+					_("Insufficient permission to connect to mpd. Check password"));
 			setup_assistant_set_error(str);
 			q_free(str);
 			return TRUE;
@@ -1146,8 +1186,8 @@ static int error_callback(MpdObj * mi, int error_id, char *error_msg, gpointer d
 			password_dialog(FALSE);
 		} else {
 			gchar *str = g_markup_printf_escaped("<b>%s %i: %s</b>",
-												 _("error code"), error_id,
-												 error_msg);
+					_("error code"), error_id,
+					error_msg);
 			playlist3_show_error_message(str, ERROR_CRITICAL);
 			g_free(str);
 		}
@@ -1185,27 +1225,16 @@ static void connection_changed(MpdObj * mi, int connected, gpointer data)
 		autoconnect_timeout = 0;
 		autoconnect_backoff = 0;
 	}
-	/**
-     * send password, first thing we do, if connected
-     */
-	/*
-	if (connected) {
-		if (connection_use_auth()) {
-			mpd_send_password(connection);
-		}
-		advanced_search_update_taglist();
-	}
-*/
 	if(connected){
 		advanced_search_update_taglist();
 	}
 	/**
-     * force an update of status, to check password
-     */
+	 * force an update of status, to check password
+	 */
 	if (connected) {
 		mpd_status_update(mi);
 		if (connected != mpd_check_connected(mi)) {
-			g_log(LOG_DOMAIN, G_LOG_LEVEL_ERROR, "State differs, exit");
+			g_log(LOG_DOMAIN, G_LOG_LEVEL_WARNING,"State differs, exit");
 			/* Probly disconnected when getting status..   exiting */
 			return;
 		}
@@ -1253,81 +1282,85 @@ static void connection_changed_real(GmpcConnection * obj, MpdObj * mi, int conne
 
 /**
  * Shows an error message.
- * When block enabled, it will run in it's own mainloop.
  */
-static void error_message_destroy(void)
+void show_error_message(const gchar * string)
 {
-	gtk_widget_destroy(error_dialog);
-	error_dialog = NULL;
-	gtk_list_store_clear(error_list_store);
+	GtkWidget *dialog = gtk_message_dialog_new_with_markup(NULL, 
+		GTK_DIALOG_MODAL,
+		GTK_MESSAGE_ERROR, 
+		GTK_BUTTONS_CLOSE,
+		"%s",string);
+	gtk_widget_show(dialog);
+	gtk_dialog_run(GTK_DIALOG(dialog));
 }
 
-void show_error_message(const gchar * string, const int block)
+
+static void move_old_gmpc_data(void)
 {
-	GtkTreeIter iter;
-	GtkWidget *label = NULL;
-	if (!error_dialog) {
-		GtkWidget *hbox = NULL, *image;
-		GtkWidget *vbox = NULL, *sw = NULL, *tree = NULL;
-		GtkWidget *pl3_win = NULL;
-		GtkCellRenderer *renderer;
-		/* create dialog */
-		error_dialog =
-			gtk_dialog_new_with_buttons(_
-										("Error occurred during operation"),
-										NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_CLOSE, GTK_RESPONSE_OK, NULL);
-		if (pl3_xml) {
-			pl3_win = playlist3_get_window();
-			gtk_window_set_transient_for(GTK_WINDOW(error_dialog), GTK_WINDOW(pl3_win));
+	gchar *url;
+	const gchar *old = g_get_home_dir();
+	gchar *path;
+
+	url =  gmpc_get_user_path(NULL);
+	if (!g_file_test(url, G_FILE_TEST_EXISTS)) {
+		path = g_build_filename(old, ".gmpc", NULL);
+		if(g_file_test(path, G_FILE_TEST_IS_DIR)) {
+			GDir *dir;
+			const gchar *iter;
+			/* Create the directory */
+			create_gmpc_paths();
+			dir = g_dir_open(path, 0, NULL);
+			if(dir){
+				while((iter = g_dir_read_name(dir)) != NULL)
+				{
+					gchar *dest_path = g_build_filename(url, iter, NULL);
+					gchar *src_path = g_build_filename(path, iter, NULL);
+					printf("move %s %s\n", src_path, dest_path);
+					g_rename(src_path, dest_path);
+					g_free(src_path); g_free(dest_path);
+				}
+				g_dir_close(dir);
+			}
 		}
-		/** create list store */
-		if (!error_list_store) {
-			error_list_store = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
-			/* don't want this destroyed */
-			g_object_ref(error_list_store);
-		}
-		hbox = gtk_hbox_new(FALSE, 6);
-		gtk_container_add(GTK_CONTAINER(GTK_DIALOG(error_dialog)->vbox), hbox);
-		gtk_container_set_border_width(GTK_CONTAINER(hbox), 9);
-
-		/* Error image */
-		image = gtk_image_new_from_stock(GTK_STOCK_DIALOG_ERROR, GTK_ICON_SIZE_DIALOG);
-		gtk_box_pack_start(GTK_BOX(hbox), image, FALSE, TRUE, 0);
-
-		vbox = gtk_vbox_new(FALSE, 6);
-		gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
-
-		/* Create label */
-		label = gtk_label_new(_("The following error(s) occurred:"));
-		gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-		gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, TRUE, 0);
-		/** Create tree view */
-		/* sw */
-		sw = gtk_scrolled_window_new(NULL, NULL);
-		gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_ETCHED_IN);
-		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-		/* tree */
-		tree = gtk_tree_view_new_with_model(GTK_TREE_MODEL(error_list_store));
-		gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tree), FALSE);
-		gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(tree), TRUE);
-		/* add tree to sw */
-		gtk_container_add(GTK_CONTAINER(sw), tree);
-		/** add cell renderers */
-		renderer = gtk_cell_renderer_text_new();
-		gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tree),
-													-1, _("Error Message"), renderer, "text", 0, NULL);
-
-		/** add sw to vbox */
-		gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
+		g_free(path);
 	}
-	gtk_list_store_append(error_list_store, &iter);
-	gtk_list_store_set(error_list_store, &iter, 0, string, -1);
-	g_signal_connect(G_OBJECT(error_dialog), "response", G_CALLBACK(error_message_destroy), NULL);
-	gtk_widget_show_all(error_dialog);
-	if (block) {
-		gtk_dialog_run(GTK_DIALOG(error_dialog));
+	g_free(url);
+
+	url =  gmpc_get_covers_path("covers.sql");
+	if (!g_file_test(url, G_FILE_TEST_EXISTS)) {
+		path = g_build_filename(old, ".covers","covers.sql", NULL);
+		if(g_file_test(path, G_FILE_TEST_EXISTS)) {
+			/* Create the directory */
+			create_gmpc_paths();
+			printf("move %s %s\n",path,url);
+			g_rename(path,url);
+		}
+		g_free(path);
+	}
+	g_free(url);
+}
+
+static void create_directory(gchar *url)
+{
+	/**
+	 * Check if ~/.gmpc/ exists
+	 * If not try to create it.
+	 */
+	if (!g_file_test(url, G_FILE_TEST_EXISTS)) {
+		if (g_mkdir_with_parents(url, 0700) < 0) {
+			g_log(LOG_DOMAIN, G_LOG_LEVEL_ERROR, "Failed to create: %s\n", url);
+			show_error_message("Failed to create config directory.");
+			abort();
+		}
+	}
+	/**
+	 * if it exists, check if it's a directory
+	 */
+	if (!g_file_test(url, G_FILE_TEST_IS_DIR)) {
+		show_error_message("The config directory is not a directory.");
+		abort();
 	} else {
-		gtk_widget_show(error_dialog);
+		g_log(LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "%s exist and is directory", url);
 	}
 }
 
@@ -1339,32 +1372,16 @@ static void create_gmpc_paths(void)
 
 	/** create path */
 	gchar *url = gmpc_get_user_path(NULL);
+	create_directory(url);
+	q_free(url);
 
-	/**
-	 * Check if ~/.gmpc/ exists
-	 * If not try to create it.
-	 */
-	if (!g_file_test(url, G_FILE_TEST_EXISTS)) {
-		if (g_mkdir_with_parents(url, 0700) < 0) {
-			g_log(LOG_DOMAIN, G_LOG_LEVEL_ERROR, "Failed to create: %s\n", url);
-			show_error_message("Failed to create config directory.", TRUE);
-			abort();
-		}
-	}
-	/**
-	 * if it exists, check if it's a directory
-	 */
-	if (!g_file_test(url, G_FILE_TEST_IS_DIR)) {
-		show_error_message("The config directory is not a directory.", TRUE);
-		abort();
-	} else {
-		g_log(LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "%s exist and is directory", url);
-	}
+	url = gmpc_get_covers_path(NULL);
+	create_directory(url);
 	/* Free the path */
 	q_free(url);
 }
 
-void print_version(void)
+static void print_version(void)
 {
 	printf(BOLD "%s\n", ("Gnome Music Player Client"));
 
