@@ -1,5 +1,5 @@
 /* Gnome Music Player Client (GMPC)
- * Copyright (C) 2004-2009 Qball Cow <qball@sarine.nl>
+ * Copyright (C) 2004-2010 Qball Cow <qball@sarine.nl>
  * Project homepage: http://gmpc.wikia.com/
  
  * This program is free software; you can redistribute it and/or modify
@@ -119,9 +119,9 @@ void mpd_song_markup_escaped(char *buffer, int size, char *markup, mpd_Song *son
  * Code copied from gnome-screenshot
  */
 
-void screenshot_add_border (GdkPixbuf **src)
+void screenshot_add_border (GdkPixbuf *src)
 {
-    GdkPixbuf *pb = *src;
+    GdkPixbuf *pb = src;
     int x,y,width,height;
     int pixel;
     int n_channels = 0;
@@ -163,15 +163,11 @@ void screenshot_add_border (GdkPixbuf **src)
  */
 gchar * gmpc_get_user_path(const gchar *filename)
 {
-	const gchar *homedir = g_get_home_dir();
+	const gchar *homedir = g_get_user_config_dir(); 
 	gchar *ret = NULL;
 
 	/* Build the path */
-#ifdef USE_CONFIG_DIR
-	ret = g_build_path(G_DIR_SEPARATOR_S, homedir,".config", "gmpc", filename,NULL);
-#else
-	ret = g_build_path(G_DIR_SEPARATOR_S, homedir, ".gmpc", filename,NULL);
-#endif
+	ret = g_build_path(G_DIR_SEPARATOR_S, homedir, "gmpc", filename,NULL);
 
 	return ret;
 }
@@ -185,18 +181,23 @@ gchar * gmpc_get_user_path(const gchar *filename)
  */
 gchar * gmpc_get_covers_path(const gchar *filename)
 {
-	const gchar *homedir = g_get_home_dir();
+	const gchar *homedir = g_get_user_cache_dir(); 
 	gchar *ret = NULL;
 
 	/* Build the path */
-#ifdef USE_CONFIG_DIR
-	ret = g_build_path(G_DIR_SEPARATOR_S, homedir,".config", "covers", filename,NULL);
-#else
-	ret = g_build_path(G_DIR_SEPARATOR_S, homedir, ".covers", filename,NULL);
-#endif
+	ret = g_build_path(G_DIR_SEPARATOR_S, homedir, "gmpc", "metadata", filename,NULL);
 	return ret;
 }
 
+gchar * gmpc_get_cache_directory(const gchar *filename)
+{
+	const gchar *homedir = g_get_user_cache_dir(); 
+	gchar *ret = NULL;
+
+	/* Build the path */
+	ret = g_build_path(G_DIR_SEPARATOR_S, homedir, "gmpc", filename,NULL);
+	return ret;
+}
 /**
  * Get's the full path to an image,
  * While this is compile time on linux, windows
