@@ -1,5 +1,5 @@
 /* Gnome Music Player Client (GMPC)
- * Copyright (C) 2004-2009 Qball Cow <qball@sarine.nl>
+ * Copyright (C) 2004-2010 Qball Cow <qball@sarine.nl>
  * Project homepage: http://gmpc.wikia.com/
 
  * This program is free software; you can redistribute it and/or modify
@@ -25,7 +25,7 @@
 #include "playlist3.h"
 #include "mpdinteraction.h"
 #include "advanced-search.h"
-#include "vala/gmpc_menu_item_rating.h"
+#include "gmpc-extras.h"
 
 //G_LOCK_DEFINE (connecting_lock);
 
@@ -41,6 +41,7 @@ static void server_pref_destroy(GtkWidget *);
 static void connection_pref_construct(GtkWidget *container);
 static void connection_pref_destroy(GtkWidget *container);
 
+void server_pref_replaygain_changed(GtkComboBox *combo, gpointer data);
 static void ServerStatusChangedCallback(MpdObj *mi, ChangedStatusType what, void *userdata);
 
 static GtkBuilder *server_pref_xml = NULL;
@@ -352,20 +353,12 @@ static void mpd_interaction_init(void)
 }
 
 gmpcPlugin server_plug = {
-	N_("Server Settings"), 	/** name */
-	{1,1,1},			/** Version */
-	GMPC_INTERNALL,		/** Plugin Type */
-	0,					/** Internal Id */
-	NULL,				/** path to plugin */
-	mpd_interaction_init,/** init */
-	NULL,                           /** Destroy */
-	NULL,				/** browser ext */
-	ServerStatusChangedCallback,	/** status changed */
-	NULL,
-	&server_gpp,			/** preferences */
-	NULL,				/** Metadata */
-	NULL,				/** get enabled */
-	NULL				/** set enabled */
+	.name = N_("Server Settings"), 	/** name */
+	.version = {1,1,1},			/** Version */
+	.plugin_type = GMPC_INTERNALL,		/** Plugin Type */
+	.init = mpd_interaction_init,/** init */
+	.mpd_status_changed = ServerStatusChangedCallback,	/** status changed */
+	.pref = &server_gpp			/** preferences */
 };
 enum
 {
@@ -382,20 +375,10 @@ gmpcPrefPlugin connection_gpp = {
 };
 
 gmpcPlugin connection_plug = {
-	N_("Connection"),       /* name */
-	{1,1,1},                /* version */
-	GMPC_INTERNALL,         /* type */
-	0,                      /* id */
-	NULL,                   /* path */
-	NULL,                   /* init function */
-	NULL,                   /* destroy function */
-	NULL,                   /* browser */
-	NULL,                   /* status changed */
-	NULL,                   /* connection changed */
-	&connection_gpp,        /* preferences */
-	NULL,                   /* metadata */
-	NULL,                   /* get_enabled */
-	NULL                    /* set_enabled */
+	.name = N_("Connection"),       /* name */
+	.version = {1,1,1},                /* version */
+	.plugin_type = GMPC_INTERNALL,         /* type */
+	.pref = &connection_gpp        /* preferences */
 };
 
 guint connecting_pulse = 0;
@@ -570,6 +553,14 @@ void repeat_toggle(void)
 {
 	mpd_player_set_repeat(connection, !mpd_player_get_repeat(connection));
 }
+void repeat_single_toggle(void)
+{
+	mpd_player_set_single(connection, !mpd_player_get_single(connection));
+}
+void consume_toggle(void)
+{
+	mpd_player_set_consume(connection, !mpd_player_get_consume(connection));
+}
 void random_toggle(void)
 {
 	mpd_player_set_random(connection, !mpd_player_get_random(connection));
@@ -854,6 +845,14 @@ void ServerStatusChangedCallback(MpdObj *mi, ChangedStatusType what, void *userd
 	{
 		update_outputs_settings();
 	}
+	if(what&MPD_CST_REPLAYGAIN)
+	{
+		if(server_pref_xml)
+		{
+			gtk_combo_box_set_active(GTK_COMBO_BOX(gtk_builder_get_object(server_pref_xml, "replay_gain_combo")),
+					mpd_server_get_replaygain_mode(connection));
+		}
+	}
 }
 
 
@@ -868,16 +867,28 @@ static void server_pref_destroy(GtkWidget *container)
 		server_pref_xml = NULL;
 	}
 }
+void server_pref_replaygain_changed(GtkComboBox *combo, gpointer data)
+{
+	unsigned int cur = gtk_combo_box_get_active(combo);
+	if(cur != mpd_server_get_replaygain_mode(connection))
+	{
+		if(mpd_server_check_command_allowed(connection, "replay_gain_mode") == MPD_SERVER_COMMAND_ALLOWED){
+			mpd_server_set_replaygain_mode(connection, cur);
+		}
+	}
+
+}
 static void server_pref_construct(GtkWidget *container)
 {
 	gchar *path = gmpc_get_full_glade_path("preferences-server.ui");
-	server_pref_xml = gtk_builder_new();//glade_xml_new(path, "server-vbox",NULL);
+	server_pref_xml = gtk_builder_new();
 	gtk_builder_add_from_file(server_pref_xml, path, NULL);
 	q_free(path);
 
 	if(server_pref_xml)
 	{
 		GtkWidget *vbox =(GtkWidget *) gtk_builder_get_object(server_pref_xml, "server-vbox");
+		GtkWidget *frame = (GtkWidget *)gtk_builder_get_object(server_pref_xml, "replay_gain_frame");
 		create_outputs_tree();
 		update_outputs_settings();
 		if(!mpd_check_connected(connection))
@@ -890,6 +901,27 @@ static void server_pref_construct(GtkWidget *container)
 			gtk_widget_set_sensitive(vbox,TRUE);
 			gtk_widget_hide((GtkWidget *)gtk_builder_get_object(server_pref_xml, "hb_warning_mesg"));
 		}
+		switch(mpd_server_check_command_allowed(connection, "replay_gain_status")){
+			case MPD_SERVER_COMMAND_ALLOWED:
+			{
+				gtk_combo_box_set_active(GTK_COMBO_BOX(gtk_builder_get_object(server_pref_xml, "replay_gain_combo")),
+					mpd_server_get_replaygain_mode(connection));
+				switch(mpd_server_check_command_allowed(connection, "replay_gain_mode")){
+					case MPD_SERVER_COMMAND_ALLOWED:
+						break;
+					default:
+						gtk_widget_set_sensitive(frame, FALSE);
+						break;
+				}
+				break;
+			}
+			case MPD_SERVER_COMMAND_NOT_ALLOWED:
+				gtk_widget_set_sensitive(frame, FALSE);
+				break;
+			case MPD_SERVER_COMMAND_NOT_SUPPORTED:
+			default:
+				gtk_widget_hide(frame);
+		}
 
 		if(mpd_status_get_crossfade(connection) == 0)
 		{
@@ -1173,6 +1205,7 @@ void connection_profiles_changed(GtkComboBox *combo, gpointer data)
 	if(gtk_combo_box_get_active_iter(combo,&iter))
 	{
 		char *value= NULL, *uid = NULL, *string;
+		const gchar *md;
 		gtk_tree_model_get(GTK_TREE_MODEL(store), &iter, 0, &uid, 1,&value, -1);
 
 
@@ -1210,9 +1243,9 @@ void connection_profiles_changed(GtkComboBox *combo, gpointer data)
 		/**
 		 * Set music directory
 		 */
-		string = gmpc_profiles_get_music_directory(gmpc_profiles, uid);
+		md = gmpc_profiles_get_music_directory(gmpc_profiles, uid);
 		gtk_entry_set_text(GTK_ENTRY((GtkWidget *) gtk_builder_get_object(xml, "music_directory")),
-				string?string:"");
+				md?md:"");
 		/**
 		 * Only enable the rmeove button when there is more then 1 profile
 		 */
@@ -1427,10 +1460,10 @@ char *connection_get_password(void)
 	g_free(profile);
 	return retv;
 }
-char *connection_get_music_directory(void)
+const char *connection_get_music_directory(void)
 {
 	gchar *profile = gmpc_profiles_get_current(gmpc_profiles);
-	gchar *retv  = gmpc_profiles_get_music_directory(gmpc_profiles, profile);
+	const gchar *retv  = gmpc_profiles_get_music_directory(gmpc_profiles, profile);
 	g_free(profile);
 	return retv;
 }
