@@ -1,5 +1,5 @@
 /* Gnome Music Player Client (GMPC)
- * Copyright (C) 2004-2009 Qball Cow <qball@sarine.nl>
+ * Copyright (C) 2004-2010 Qball Cow <qball@sarine.nl>
  * Project homepage: http://gmpc.wikia.com/
  
  * This program is free software; you can redistribute it and/or modify
@@ -30,11 +30,8 @@
 /* every part split out over multiple files */
 #include "revision.h"
 #include "gmpc-clicklabel.h"
-#include "vala/gmpc-liststore-sort.h"
 #include "gmpc-metaimage.h"
-#include "vala/gmpc-progress.h"
-#include "vala/gmpc-favorites.h"
-#include "vala/gmpc-metadata-browser2.h"
+#include "gmpc-extras.h"
 #ifdef ENABLE_IGE
 #include "ige-mac-menu.h"
 #include "ige-mac-dock.h"
@@ -47,7 +44,10 @@
 #include "playlist3-keybindings.h"
 
 #define ALBUM_SIZE_SMALL 48
-#define ALBUM_SIZE_LARGE 72
+#define ALBUM_SIZE_LARGE 80
+
+#define LOG_DOMAIN "Playlist"
+
 /* Drag and drop Target table */
 static GtkTargetEntry target_table[] = {
 	{(gchar *) "x-url/http", 0, 0},
@@ -142,12 +142,15 @@ static GtkBuilder *playlist_pref_xml = NULL;
 
 static GtkWidget *volume_button = NULL;
 
+void ck_search_as_you_type(GtkToggleButton * but);
 /**
  * Status icons
  */
 
 static void main_window_update_status_icons(void);
 static GtkWidget *si_repeat = NULL;
+static GtkWidget *si_consume = NULL;
+static GtkWidget *si_repeat_single = NULL;
 static GtkWidget *si_random = NULL;
 
 /* Get the type of the selected row..
@@ -503,7 +506,7 @@ int pl3_window_key_press_event(GtkWidget * mw, GdkEventKey * event)
 					else if (action == KB_ACTION_TOGGLE_MUTE)
 						volume_toggle_mute();
 					else {
-						debug_printf(DEBUG_ERROR,
+						g_log(LOG_DOMAIN, G_LOG_LEVEL_WARNING,
 								"Keybinding action (%i) for: %i %i is invalid\n", action, event->state, event->keyval);
 						found = 0;
 					}
@@ -559,14 +562,23 @@ gboolean pl3_close(void)
 {
 	/* only save when window is PLAYLIST_SMALL or NO ZOOM */
 	if (pl3_xml != NULL) {
-		gtk_window_get_position(GTK_WINDOW(playlist3_get_window()), &pl3_wsize.x, &pl3_wsize.y);
+		GtkWidget *window = playlist3_get_window();
+		int maximized = FALSE;
+		if(window->window)
+		{
+			GdkWindowState state = gdk_window_get_state(window->window);
+			maximized = ((state&GDK_WINDOW_STATE_MAXIMIZED) > 0);
+		}
+		cfg_set_single_value_as_int(config, "playlist", "maximized", maximized);
+
+		gtk_window_get_position(GTK_WINDOW(window), &pl3_wsize.x, &pl3_wsize.y);
 
 		cfg_set_single_value_as_int(config, "playlist", "xpos", pl3_wsize.x);
 		cfg_set_single_value_as_int(config, "playlist", "ypos", pl3_wsize.y);
 
 		if (pl3_zoom <= PLAYLIST_SMALL) {
-			gtk_window_get_size(GTK_WINDOW(playlist3_get_window()), &pl3_wsize.width, &pl3_wsize.height);
-			debug_printf(DEBUG_INFO, "pl3_close: save size: %i %i\n", pl3_wsize.width, pl3_wsize.height);
+			gtk_window_get_size(GTK_WINDOW(window), &pl3_wsize.width, &pl3_wsize.height);
+			g_log(LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "pl3_close: save size: %i %i\n", pl3_wsize.width, pl3_wsize.height);
 			cfg_set_single_value_as_int(config, "playlist", "width", pl3_wsize.width);
 			cfg_set_single_value_as_int(config, "playlist", "height", pl3_wsize.height);
 		}
@@ -604,6 +616,14 @@ int pl3_hide(void)
 		return 1;
 	}
 	if (pl3_xml != NULL && !pl3_hidden) {
+		GtkWidget *window = playlist3_get_window();
+		int maximized = FALSE;
+		if(window->window)
+		{
+			GdkWindowState state = gdk_window_get_state(window->window);
+			maximized = ((state&GDK_WINDOW_STATE_MAXIMIZED) > 0);
+		}
+		cfg_set_single_value_as_int(config, "playlist", "maximized", maximized);
 		/** Save position
 		*/
 		gtk_window_get_position(GTK_WINDOW(pl3_win), &pl3_wsize.x, &pl3_wsize.y);
@@ -613,7 +633,7 @@ int pl3_hide(void)
 		 */
 		if (pl3_zoom <= PLAYLIST_SMALL) {
 			gtk_window_get_size(GTK_WINDOW(pl3_win), &pl3_wsize.width, &pl3_wsize.height);
-			debug_printf(DEBUG_INFO, "pl3_hide: save size: %i %i\n", pl3_wsize.width, pl3_wsize.height);
+			g_log(LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "pl3_hide: save size: %i %i\n", pl3_wsize.width, pl3_wsize.height);
 			cfg_set_single_value_as_int(config, "playlist", "width", pl3_wsize.width);
 			cfg_set_single_value_as_int(config, "playlist", "height", pl3_wsize.height);
 			cfg_set_single_value_as_int(config, "playlist", "pane-pos",
@@ -638,7 +658,7 @@ static void pl3_show_and_position_window(void)
 		gtk_window_move(GTK_WINDOW(pl3_win), pl3_wsize.x, pl3_wsize.y);
 	}
 	if (pl3_wsize.height > 0 && pl3_wsize.width > 0) {
-		debug_printf(DEBUG_INFO, "restore size %i %i\n", pl3_wsize.width, pl3_wsize.height);
+		g_log(LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "restore size %i %i\n", pl3_wsize.width, pl3_wsize.height);
 		gtk_window_resize(GTK_WINDOW(pl3_win), pl3_wsize.width, pl3_wsize.height);
 	}
 	gtk_widget_show(pl3_win);
@@ -790,14 +810,31 @@ static void playlist_connection_changed(MpdObj * mi, int connect, gpointer data)
 		if (gtk_tree_model_get_iter_first(model, &iter)) {
 			gtk_tree_selection_select_iter(selec, &iter);
 		}
-		string = g_strdup_printf("%s - %s %s", _("GMPC"), _("Connected to"), mpd_get_hostname(mi));
+		if(gmpc_profiles_get_number_of_profiles(gmpc_profiles) >1){
+			gchar *id = gmpc_profiles_get_current(gmpc_profiles);
+			if(id) {
+				string = g_strdup_printf("[%s] %s - %s %s",gmpc_profiles_get_name(gmpc_profiles, id), _("GMPC"), _("Connected to"), mpd_get_hostname(mi));
+				g_free(id);
+			}
+		}
+		if(!string)
+			string = g_strdup_printf("%s - %s %s", _("GMPC"), _("Connected to"), mpd_get_hostname(mi));
 		gtk_window_set_title(GTK_WINDOW(pl3_win), string);
-		q_free(string);
+		g_free(string);
 	} else {
 		gchar *string = NULL;
-		string = g_strdup_printf("%s - %s", _("GMPC"), _("Disconnected"));
+
+		if(gmpc_profiles_get_number_of_profiles(gmpc_profiles) >1){
+			gchar *id = gmpc_profiles_get_current(gmpc_profiles);
+			if(id) {
+				string = g_strdup_printf("[%s] %s - %s",gmpc_profiles_get_name(gmpc_profiles, id), _("GMPC"), _("Disconnected"));
+				g_free(id);
+			}
+		}
+		if(!string)
+			string = g_strdup_printf("%s - %s", _("GMPC"), _("Disconnected"));
 		gtk_window_set_title(GTK_WINDOW(pl3_win), string);
-		q_free(string);
+		g_free(string);
 	}
 
 	/*
@@ -861,15 +898,11 @@ void create_playlist3(void)
 	 * Check if the file is loaded, if not then show an error message and abort the program
 	 */
 	if (pl3_xml == NULL) {
-		char *errormsg = g_strdup_printf("%s: '%s'",
-										 _("Failed to open the interface description file!\nPlease reinstall gmpc\n"),
-										 path);
-		debug_printf(DEBUG_ERROR, "Failed to open playlist3.glade.\n");
-		show_error_message(errormsg, TRUE);
+		g_log(LOG_DOMAIN, G_LOG_LEVEL_WARNING, "Failed to open playlist3.glade.\n");
 		abort();
 	}
 
-	q_free(path);
+	g_free(path);
 	/** murrine hack */
 	if (cfg_get_single_value_as_int_with_default(config, "Default", "murrine-hack", FALSE)) {
 		GdkScreen *screen;
@@ -944,6 +977,13 @@ void create_playlist3(void)
 	column = gtk_tree_view_column_new();
 	gtk_tree_view_column_pack_start(column, renderer, FALSE);
 	g_object_set(G_OBJECT(renderer),  "stock-size", GTK_ICON_SIZE_DND, NULL);
+	{
+		int w,h;
+		if(gtk_icon_size_lookup(GTK_ICON_SIZE_DND, &w, &h))
+		{
+			g_object_set(G_OBJECT(renderer),  "height",h,  NULL);
+		}
+	}
 	gtk_tree_view_column_set_attributes(column, renderer, "icon-name", PL3_CAT_ICON_ID, NULL);
 
 	renderer = gtk_cell_renderer_text_new();
@@ -1063,6 +1103,7 @@ void create_playlist3(void)
 
 	/* restore the window's position and size, if the user wants this. */
 	if (cfg_get_single_value_as_int_with_default(config, "playlist", "savepossize", 0)) {
+		int maximized = cfg_get_single_value_as_int_with_default(config, "playlist", "maximized", 0);
 		/* Load values from config file */
 		pl3_wsize.x = cfg_get_single_value_as_int_with_default(config, "playlist", "xpos", 0);
 		pl3_wsize.y = cfg_get_single_value_as_int_with_default(config, "playlist", "ypos", 0);
@@ -1075,7 +1116,7 @@ void create_playlist3(void)
 			gtk_window_move(GTK_WINDOW(playlist3_get_window()), pl3_wsize.x, pl3_wsize.y);
 		}
 		if (pl3_wsize.height > 0 && pl3_wsize.width > 0) {
-			debug_printf(DEBUG_INFO, "restore size %i %i\n", pl3_wsize.width, pl3_wsize.height);
+			g_log(LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "restore size %i %i\n", pl3_wsize.width, pl3_wsize.height);
 			gtk_window_resize(GTK_WINDOW(playlist3_get_window()), pl3_wsize.width, pl3_wsize.height);
 		}
 		/* restore pane position */
@@ -1084,6 +1125,7 @@ void create_playlist3(void)
 								   (glade_xml_get_widget(pl3_xml, "hpaned1")),
 								   cfg_get_single_value_as_int(config, "playlist", "pane-pos"));
 		}
+		if(maximized)gtk_window_maximize(GTK_WINDOW(playlist3_get_window()));
 		/**
 		 * restore zoom level
 		 */
@@ -1209,6 +1251,8 @@ void create_playlist3(void)
 	gtk_widget_show_all(si_repeat);
 	main_window_add_status_icon(si_repeat);
 
+
+
 	si_random = gtk_event_box_new();
 	g_signal_connect(G_OBJECT(si_random), "button-release-event", G_CALLBACK(random_toggle), NULL);
 	gtk_container_add(GTK_CONTAINER(si_random), 
@@ -1216,6 +1260,20 @@ void create_playlist3(void)
 	gtk_widget_show_all(si_random);
 	main_window_add_status_icon(si_random);
 
+	si_repeat_single = gtk_event_box_new();
+	g_signal_connect(G_OBJECT(si_repeat_single), "button-release-event", G_CALLBACK(repeat_single_toggle), NULL);
+	gtk_container_add(GTK_CONTAINER(si_repeat_single), 
+			gtk_image_new_from_icon_name("media-repeat-single", GTK_ICON_SIZE_MENU));
+	gtk_widget_show_all(si_repeat_single);
+	main_window_add_status_icon(si_repeat_single);
+
+	si_consume = gtk_event_box_new();
+	g_signal_connect(G_OBJECT(si_consume), "button-release-event", G_CALLBACK(consume_toggle), NULL);
+	gtk_container_add(GTK_CONTAINER(si_consume), 
+			gtk_image_new_from_icon_name("media-consume", GTK_ICON_SIZE_MENU));
+	gtk_widget_show_all(si_consume);
+	main_window_add_status_icon(si_consume);
+
 	/* Listen for icon changed 
 	g_object_connect(gtk_icon_theme_get_default(), "changed",
 			G_CALLBACK(main_window_update_status_icons), NULL);
@@ -1304,6 +1362,11 @@ G_MODULE_EXPORT void ck_show_tabbed_heading_enable_cb(GtkToggleButton * but)
 	}
 	cfg_set_single_value_as_int(config, "playlist", "button-heading", bool1);
 }
+G_MODULE_EXPORT void ck_search_as_you_type(GtkToggleButton * but)
+{
+	int bool1 = gtk_toggle_button_get_active(but);
+	cfg_set_single_value_as_int(config, "general", "search-as-you-type", bool1);
+}
 
 static void playlist_pref_destroy(GtkWidget * container)
 {
@@ -1362,10 +1425,14 @@ void playlist_pref_construct(GtkWidget * container)
 									 (config, "playlist", "button-heading", FALSE)
 			);
 
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON
+									 (gtk_builder_get_object
+									  (playlist_pref_xml, "ck_search_as_you_type")),
+									 cfg_get_single_value_as_int_with_default(config, "general", "search-as-you-type", 0));
 		gtk_container_add(GTK_CONTAINER(container), vbox);
 		gtk_builder_connect_signals(playlist_pref_xml, NULL);
 	}
-	q_free(path);
+	g_free(path);
 }
 
 
@@ -1451,7 +1518,7 @@ static void playlist_zoom_level_changed(void)
 		gtk_window_get_size(GTK_WINDOW(pl3_win), &pl3_wsize.width, &pl3_wsize.height);
 		cfg_set_single_value_as_int(config, "playlist", "width", pl3_wsize.width);
 		cfg_set_single_value_as_int(config, "playlist", "height", pl3_wsize.height);
-		debug_printf(DEBUG_INFO, "save size: %i %i\n", pl3_wsize.width, pl3_wsize.height);
+		g_log(LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "save size: %i %i\n", pl3_wsize.width, pl3_wsize.height);
 	}
 
 	if (pl3_old_zoom == PLAYLIST_MINI && pl3_zoom != PLAYLIST_MINI) {
@@ -1499,7 +1566,7 @@ static void playlist_zoom_level_changed(void)
 
 	gtk_window_set_resizable(GTK_WINDOW(pl3_win), TRUE);
 	if (pl3_wsize.width > 0 && pl3_wsize.height > 0 && pl3_old_zoom == PLAYLIST_MINI) {
-		debug_printf(DEBUG_INFO, "restore size %i %i\n", pl3_wsize.width, pl3_wsize.height);
+		g_log(LOG_DOMAIN, G_LOG_LEVEL_DEBUG, "restore size %i %i\n", pl3_wsize.width, pl3_wsize.height);
 		gtk_window_resize(GTK_WINDOW(pl3_win), pl3_wsize.width, pl3_wsize.height);
 	}
 	gtk_widget_show(glade_xml_get_widget(pl3_xml, "vbox5"));
@@ -1593,7 +1660,19 @@ static void playlist_status_changed(MpdObj * mi, ChangedStatusType what, void *u
 					 * Update window title
 					 */
 					mpd_song_markup(buffer, 1024, markup, mpd_playlist_get_current_song(connection));
-					gtk_window_set_title(GTK_WINDOW(pl3_win), buffer);
+
+					if(gmpc_profiles_get_number_of_profiles(gmpc_profiles) >1){
+						gchar *id = gmpc_profiles_get_current(gmpc_profiles);
+						if(id) {
+							gchar *string = g_strdup_printf("[%s] %s",gmpc_profiles_get_name(gmpc_profiles,
+							id),buffer);
+							gtk_window_set_title(GTK_WINDOW(pl3_win), string);
+							g_free(id);
+							g_free(string);
+						}
+					}
+					else
+						gtk_window_set_title(GTK_WINDOW(pl3_win), buffer);
 
 					g_free(markup);
 
@@ -1602,7 +1681,7 @@ static void playlist_status_changed(MpdObj * mi, ChangedStatusType what, void *u
 					if (pb) {
 						ige_mac_dock_set_icon_from_pixbuf(dock, pb);
 					} else {
-						debug_printf(DEBUG_ERROR, "failed to get icon\n");
+						g_log(LOG_DOMAIN, G_LOG_LEVEL_WARNING, "failed to get icon\n");
 					}
 #endif
 					break;
@@ -1631,13 +1710,25 @@ static void playlist_status_changed(MpdObj * mi, ChangedStatusType what, void *u
 					strcat(buffer, " (");
 					strcat(buffer, _("paused"));
 					strcat(buffer, ")");
-					gtk_window_set_title(GTK_WINDOW(pl3_win), buffer);
+
+					if(gmpc_profiles_get_number_of_profiles(gmpc_profiles) >1){
+						gchar *id = gmpc_profiles_get_current(gmpc_profiles);
+						if(id) {
+							gchar *string = g_strdup_printf("[%s] %s",gmpc_profiles_get_name(gmpc_profiles,
+							id),buffer);
+							gtk_window_set_title(GTK_WINDOW(pl3_win), string);
+							g_free(id);
+							g_free(string);
+						}
+					}
+					else
+						gtk_window_set_title(GTK_WINDOW(pl3_win), buffer);
 #ifdef ENABLE_IGE
 					pb = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(), "gmpc-tray-pause", 64, 0, NULL);
 					if (pb) {
 						ige_mac_dock_set_icon_from_pixbuf(dock, pb);
 					} else {
-						debug_printf(DEBUG_ERROR, "failed to get icon\n");
+						g_log(LOG_DOMAIN, G_LOG_LEVEL_WARNING, "failed to get icon\n");
 					}
 #endif
 					g_free(markup);
@@ -1653,13 +1744,24 @@ static void playlist_status_changed(MpdObj * mi, ChangedStatusType what, void *u
 						(glade_xml_get_widget
 						 (pl3_xml, "pp_but_play_img")), "gtk-media-play", GTK_ICON_SIZE_BUTTON);
 
-				gtk_window_set_title(GTK_WINDOW(pl3_win), _("GMPC"));
+				if(gmpc_profiles_get_number_of_profiles(gmpc_profiles) >1){
+					gchar *id = gmpc_profiles_get_current(gmpc_profiles);
+					if(id) {
+						gchar *string = g_strdup_printf("[%s] %s",gmpc_profiles_get_name(gmpc_profiles,
+									id),_("GMPC"));
+						gtk_window_set_title(GTK_WINDOW(pl3_win),string); 
+						g_free(id);
+						g_free(string);
+					}
+				}
+				else
+					gtk_window_set_title(GTK_WINDOW(pl3_win), _("GMPC"));
 #ifdef ENABLE_IGE
 				pb = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(), "gmpc", 64, 0, NULL);
 				if (pb) {
 					ige_mac_dock_set_icon_from_pixbuf(dock, pb);
 				} else {
-					debug_printf(DEBUG_ERROR, "failed to get icon\n");
+					g_log(LOG_DOMAIN, G_LOG_LEVEL_WARNING, "failed to get icon\n");
 				}
 #endif
 
@@ -1689,7 +1791,7 @@ static void playlist_status_changed(MpdObj * mi, ChangedStatusType what, void *u
 			char *string = g_strdup_printf(_("Repeat: %s"),
 										   (mpd_player_get_repeat(connection)) ? _("On") : _("Off"));
 			pl3_push_statusbar_message(string);
-			q_free(string);
+			g_free(string);
 
 			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
 										   (glade_xml_get_widget
@@ -1702,14 +1804,14 @@ static void playlist_status_changed(MpdObj * mi, ChangedStatusType what, void *u
 			char *string = g_strdup_printf(_("Random: %s"),
 										   (mpd_player_get_random(connection)) ? _("On") : _("Off"));
 			pl3_push_statusbar_message(string);
-			q_free(string);
+			g_free(string);
 
 			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
 										   (glade_xml_get_widget
 											(pl3_xml, "menu_random")), mpd_player_get_random(connection));
 		}
 	}
-	if(what&(MPD_CST_RANDOM|MPD_CST_REPEAT)) {
+	if(what&(MPD_CST_RANDOM|MPD_CST_REPEAT|MPD_CST_SINGLE_MODE|MPD_CST_CONSUME_MODE)) {
 		main_window_update_status_icons();
 	}
 	if (what & MPD_CST_SINGLE_MODE) {
@@ -1717,7 +1819,7 @@ static void playlist_status_changed(MpdObj * mi, ChangedStatusType what, void *u
 			char *string = g_strdup_printf(_("Single mode: %s"),
 										   (mpd_player_get_single(connection)) ? _("On") : _("Off"));
 			pl3_push_statusbar_message(string);
-			q_free(string);
+			g_free(string);
 
 			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
 										   (glade_xml_get_widget
@@ -1730,7 +1832,7 @@ static void playlist_status_changed(MpdObj * mi, ChangedStatusType what, void *u
 			char *string = g_strdup_printf(_("Consume: %s"),
 										   (mpd_player_get_consume(connection)) ? _("On") : _("Off"));
 			pl3_push_statusbar_message(string);
-			q_free(string);
+			g_free(string);
 
 			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM
 										   (glade_xml_get_widget
@@ -1791,8 +1893,8 @@ static void playlist_status_changed(MpdObj * mi, ChangedStatusType what, void *u
 												 _("MPD Reported the following error"),
 												 error);
 			playlist3_show_error_message(mes, ERROR_WARNING);
-			q_free(mes);
-			q_free(error);
+			g_free(mes);
+			g_free(error);
 		}
 	}
 	if (what & MPD_CST_OUTPUT) {
@@ -1844,7 +1946,7 @@ void about_window(void)
 
 	gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(playlist3_get_window()));
 	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER_ON_PARENT);
-	q_free(path);
+	g_free(path);
 
 	if (strlen(revision)) {
 		path = g_strdup_printf("%s\nRevision: %s", VERSION, revision);
@@ -1852,10 +1954,11 @@ void about_window(void)
 		path = g_strdup_printf("%s\n%s\n", VERSION, GMPC_TAGLINE);
 	}
 	gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(dialog), GMPC_COPYRIGHT);
+	gtk_about_dialog_set_program_name(GTK_ABOUT_DIALOG(dialog), _("Gnome Music Player Client"));
 	gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(dialog), GMPC_WEBSITE);
 	gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(dialog), path);
 
-	q_free(path);
+	g_free(path);
 	gtk_widget_show(dialog);
 	gtk_dialog_run(GTK_DIALOG(dialog));
 	gtk_widget_destroy(dialog);
@@ -1934,7 +2037,9 @@ static void pl3_profiles_changed(GmpcProfiles * prof, const int changed, const i
 		pl3_push_statusbar_message(message);
 		g_free(message);
 	}
-
+	if(!mpd_check_connected(connection)) {
+		playlist_connection_changed(connection, 0, NULL);
+	}
 }
 
 static void pl3_update_profiles_menu(GmpcProfiles * prof, const int changed, const int col, const gchar * id)
@@ -1946,7 +2051,7 @@ static void pl3_update_profiles_menu(GmpcProfiles * prof, const int changed, con
 	/* check if there is anything changed that is important for us. */
 
 	if (changed == PROFILE_COL_CHANGED && col != PROFILE_COL_NAME) {
-		q_free(current);
+		g_free(current);
 		return;
 	}
 	/***
@@ -2127,11 +2232,6 @@ static void playlist3_header_artist(void)
 	}
 }
 
-static gboolean playlist3_header_artist_tooltip(GtkWidget * label, gint x,
-												gint y, gboolean keyboard_mode, GtkTooltip * tip, gpointer data)
-{
-	return TRUE;
-}
 
 static void playlist3_header_album(void)
 {
@@ -2177,21 +2277,6 @@ void playlist3_new_header(void)
 		g_signal_connect(G_OBJECT(header_labels[0]), "clicked", G_CALLBACK(playlist3_header_song), NULL);
 		g_signal_connect(G_OBJECT(header_labels[2]), "clicked", G_CALLBACK(playlist3_header_artist), NULL);
 		g_signal_connect(G_OBJECT(header_labels[4]), "clicked", G_CALLBACK(playlist3_header_album), NULL);
-		/* Create tooltip */
-		{
-			GtkWidget *win, *item = (GtkWidget *) gmpc_metaimage_new_size(META_ARTIST_ART, 200);
-			gmpc_metaimage_set_squared(GMPC_METAIMAGE(item), FALSE);
-			gmpc_metaimage_set_connection(GMPC_METAIMAGE(item), connection);
-			gmpc_metaimage_set_hide_on_na(GMPC_METAIMAGE(item), TRUE);
-			win = gtk_window_new(GTK_WINDOW_POPUP);
-			gtk_window_set_resizable(GTK_WINDOW(win), FALSE);
-			gtk_container_add(GTK_CONTAINER(win), item);
-			gtk_widget_set_has_tooltip(GTK_WIDGET(header_labels[2]), TRUE);
-			gtk_widget_show_all(item);
-			gtk_widget_set_tooltip_window(GTK_WIDGET(header_labels[2]), GTK_WINDOW(win));
-			g_signal_connect(G_OBJECT(header_labels[2]), "query-tooltip",
-							 G_CALLBACK(playlist3_header_artist_tooltip), NULL);
-		}
 		gtk_box_pack_start(GTK_BOX(hbox10), vbox, TRUE, TRUE, 0);
 		gtk_widget_show_all(hbox10);
 	}
@@ -2274,7 +2359,7 @@ static gboolean playlist3_error_expose(GtkWidget * wid, GdkEventExpose * event,
 	gtk_paint_flat_box(wid->style,
 					   wid->window,
 					   GTK_STATE_NORMAL,
-					   GTK_SHADOW_OUT, NULL, wid, "tooltip", 0, 0, wid->allocation.width-12, wid->allocation.height-12);
+					   GTK_SHADOW_OUT, NULL, wid, "tooltip", 0, 0, wid->allocation.width, wid->allocation.height);
 
 	return FALSE;
 }
@@ -2311,7 +2396,7 @@ gboolean playlist3_show_playtime(gulong playtime)
 	if (playtime) {
 		gchar *string = format_time(playtime);
 		pl3_push_rsb_message(string);
-		q_free(string);
+		g_free(string);
 	} else {
 		pl3_push_rsb_message("");
 	}
@@ -2608,6 +2693,28 @@ void easy_command_help_window(void)
 
 static void main_window_update_status_icons(void)
 {
+	if(si_repeat_single)
+	{
+		GtkWidget *image = gtk_bin_get_child(GTK_BIN(si_repeat_single));
+		if(mpd_check_connected(connection) && mpd_player_get_single(connection)){
+			gtk_widget_set_sensitive(GTK_WIDGET(image), TRUE);
+			gtk_widget_set_tooltip_text(si_repeat_single, _("Single Mode enabled"));
+		}else{
+			gtk_widget_set_sensitive(GTK_WIDGET(image), FALSE);
+			gtk_widget_set_tooltip_text(si_repeat_single, _("Single Mode disabled"));
+		}
+	}
+	if(si_consume)
+	{
+		GtkWidget *image = gtk_bin_get_child(GTK_BIN(si_consume));
+		if(mpd_check_connected(connection) && mpd_player_get_consume(connection)){
+			gtk_widget_set_sensitive(GTK_WIDGET(image), TRUE);
+			gtk_widget_set_tooltip_text(si_consume, _("Consume Mode enabled"));
+		}else{
+			gtk_widget_set_sensitive(GTK_WIDGET(image), FALSE);
+			gtk_widget_set_tooltip_text(si_consume, _("Consume Mode disabled"));
+		}
+	}
 	if(si_repeat)
 	{
 		GtkWidget *image = gtk_bin_get_child(GTK_BIN(si_repeat));
