@@ -1,5 +1,5 @@
 /* Gnome Music Player Client (GMPC)
- * Copyright (C) 2004-2009 Qball Cow <qball@sarine.nl>
+ * Copyright (C) 2004-2010 Qball Cow <qball@sarine.nl>
  * Project homepage: http://gmpc.wikia.com/
  
  * This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,7 @@
 #include <glib.h>
 #include <gtk/gtk.h>
 #include <gmodule.h>
-#include "vala/gmpc-plugin.h"
+#include "gmpc-extras.h"
 #include "main.h"
 #include "metadata.h"
 
@@ -42,10 +42,10 @@ gmpcPluginParent * plugin_get_from_id(int id)
 	return plugins[pos];
 }
 
-static inline GQuark
+static GQuark
 plugin_quark(void)
 {
-	return g_quark_from_static_string("tagedit_plugin");
+	return g_quark_from_static_string("gmpc_plugin");
 }
 
 int plugin_get_pos(int id)
@@ -172,7 +172,7 @@ void plugin_add(gmpcPlugin *plug, int plugin, GError **error)
 	plugins[num_plugins-1] = parent;
 	plugins[num_plugins] = NULL;
 	
-	if(plug->plugin_type&GMPC_PLUGIN_META_DATA)
+	if(gmpc_plugin_is_metadata(parent))//plug->plugin_type&GMPC_PLUGIN_META_DATA)
 	{
 		meta_data_add_plugin(parent);
 	}
@@ -593,12 +593,11 @@ int gmpc_plugin_browser_song_list_option_menu(gmpcPluginParent *plug, GmpcMpdDat
 
 gboolean gmpc_plugin_browser_has_integrate_search(gmpcPluginParent *plug)
 {
+    if(plug->new){
+        return GMPC_PLUGIN_IS_INTEGRATE_SEARCH_IFACE(plug->new);
+    }
     if(gmpc_plugin_is_browser(plug))
     {
-        if(plug->new){
-            /* TODO */
-            return FALSE;
-        }
         return plug->old->browser->integrate_search != NULL;
     }
     return FALSE;
@@ -606,12 +605,16 @@ gboolean gmpc_plugin_browser_has_integrate_search(gmpcPluginParent *plug)
 MpdData *gmpc_plugin_browser_integrate_search(gmpcPluginParent *plug, const int search_field, const gchar * query, GError **error)
 {
     if(!gmpc_plugin_browser_has_integrate_search(plug)) return NULL;
+    if(plug->new) return gmpc_plugin_integrate_search_iface_search(GMPC_PLUGIN_INTEGRATE_SEARCH_IFACE(plug->new), search_field, query);
     return plug->old->browser->integrate_search(search_field,query,error); 
 }
 
 gboolean gmpc_plugin_browser_integrate_search_field_supported(gmpcPluginParent *plug, const int search_field)
 {
     if(!gmpc_plugin_browser_has_integrate_search(plug)) return FALSE;
+    
+    if(plug->new) return gmpc_plugin_integrate_search_iface_field_supported(GMPC_PLUGIN_INTEGRATE_SEARCH_IFACE(plug->new), search_field);
+
 	if(plug->old->browser->integrate_search_field_supported == NULL) return TRUE;
     return plug->old->browser->integrate_search_field_supported(search_field); 
 }
@@ -720,7 +723,7 @@ void gmpc_plugin_metadata_query_metadata_list(gmpcPluginParent *plug, mpd_Song *
     {
         if(plug->new)
         {
-            gmpc_plugin_meta_data_iface_get_data(GMPC_PLUGIN_META_DATA_IFACE(plug->new), song, type, callback, data);
+            gmpc_plugin_meta_data_iface_get_metadata(GMPC_PLUGIN_META_DATA_IFACE(plug->new), song, type, callback, data);
             return;
         }
         if(plug->old->metadata->get_metadata)
