@@ -1,5 +1,5 @@
 /* Gnome Music Player Client (GMPC)
- * Copyright (C) 2004-2009 Qball Cow <qball@sarine.nl>
+ * Copyright (C) 2004-2010 Qball Cow <qball@sarine.nl>
  * Project homepage: http://gmpc.wikia.com/
  
  * This program is free software; you can redistribute it and/or modify
@@ -23,11 +23,11 @@
 #include <config.h>
 #include "main.h"
 #include "playlist3.h"
-#include "vala/gmpc-progress.h"
+#include "gmpc-extras.h"
 #include "gmpc-metaimage.h"
 #include "misc.h"
 
-
+#define LOG_DOMAIN "TrayIcon"
 /* name of config field */
 #define TRAY_ICON2_ID "tray-icon2"
 
@@ -197,6 +197,7 @@ static int tray_icon2_button_scroll_event(gpointer tray, GdkEventScroll *event,
     return TRUE;
 }
 /* hack to delay tooltip showup on tray-icon*/
+/*
 GTimeVal current = {0,0};
 static gboolean tray_icon2_tooltip_query(GtkStatusIcon *icon, 
         gint x, gint y, 
@@ -228,7 +229,7 @@ static gboolean tray_icon2_tooltip_query(GtkStatusIcon *icon,
     }
     return FALSE;
 }
-
+*/
 static void tray_icon2_init(void)
 {
 
@@ -248,7 +249,7 @@ static void tray_icon2_init(void)
 			g_signal_connect(G_OBJECT(tray_icon2_gsi), "button-press-event", G_CALLBACK(tray_icon2_button_press_event), NULL);
 			g_signal_connect(G_OBJECT(tray_icon2_gsi), "scroll-event", G_CALLBACK(tray_icon2_button_scroll_event), NULL);
 
-            g_signal_connect(G_OBJECT(tray_icon2_gsi), "query-tooltip", G_CALLBACK(tray_icon2_tooltip_query), NULL);
+        //    g_signal_connect(G_OBJECT(tray_icon2_gsi), "query-tooltip", G_CALLBACK(tray_icon2_tooltip_query), NULL);
 		}
         /* Make sure the icons are up2date */
         tray_icon2_connection_changed(connection, mpd_check_connected(connection), NULL);
@@ -376,6 +377,21 @@ static gboolean tray_icon2_tooltip_button_press_event(GtkWidget *box, GdkEventBu
     return TRUE;
 }
 
+static gboolean popup_enter_notify_event(GtkWidget *event, GdkEventCrossing *eventc, gpointer data)
+{
+    if(tray_icon2_tooltip_timeout)
+    {
+        g_source_remove(tray_icon2_tooltip_timeout);
+    }
+    tray_icon2_tooltip_timeout = 0; 
+    return FALSE;
+}
+static gboolean popup_leave_notify_event(GtkWidget *event, GdkEventCrossing *eventc, gpointer data)
+{
+    int tooltip_timeout = cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "tooltip-timeout", 5);
+    tray_icon2_tooltip_timeout = g_timeout_add_seconds(tooltip_timeout, (GSourceFunc)tray_icon2_tooltip_destroy, NULL);
+    return FALSE;
+}
 
 static void tray_icon2_create_tooltip_real(int position)
 {
@@ -493,6 +509,10 @@ static void tray_icon2_create_tooltip_real(int position)
     gtk_widget_modify_bg(GTK_WIDGET(event), GTK_STATE_NORMAL, &(pl3_win->style->bg[GTK_STATE_SELECTED]));
     gtk_container_add(GTK_CONTAINER(event), coverimg);
 	gtk_box_pack_start(GTK_BOX(hbox), event, FALSE,TRUE,0);
+
+
+    g_signal_connect(G_OBJECT(tray_icon2_tooltip), "enter-notify-event",  G_CALLBACK(popup_enter_notify_event), NULL);
+    g_signal_connect(G_OBJECT(tray_icon2_tooltip), "leave-notify-event",  G_CALLBACK(popup_leave_notify_event), NULL);
 	/**
 	 * 2
 	 * 
@@ -717,7 +737,7 @@ static void tray_icon2_status_changed(MpdObj *mi, ChangedStatusType what, void *
             new_checksum = mpd_song_checksum(song);
 			if(state == MPD_PLAYER_PLAY || state == MPD_PLAYER_PAUSE)
 			{
-                if(current_song_checksum == NULL || strcmp(current_song_checksum, new_checksum))
+                if(new_checksum == NULL || current_song_checksum == NULL || strcmp(current_song_checksum, new_checksum))
                     tray_icon2_create_tooltip();
             }
             if(current_song_checksum){
@@ -804,7 +824,7 @@ static void tray_icon2_connection_changed(MpdObj *mi, int connect,void *user_dat
 
 void tray_enable_toggled(GtkToggleButton *but)
 {
-	debug_printf(DEBUG_INFO,"tray-icon.c: changing tray icon %i\n", gtk_toggle_button_get_active(but));
+	g_log(LOG_DOMAIN, G_LOG_LEVEL_DEBUG,"tray-icon.c: changing tray icon %i\n", gtk_toggle_button_get_active(but));
 	cfg_set_single_value_as_int(config, TRAY_ICON2_ID, "enable", (int)gtk_toggle_button_get_active(but));
 	if(cfg_get_single_value_as_int_with_default(config, TRAY_ICON2_ID, "enable", 1)) {
 		tray_icon2_set_enabled(TRUE);
